searchData={"items":[{"type":"module","title":"K8s","doc":"Kubernetes API Client for Elixir","ref":"K8s.html"},{"type":"function","title":"K8s.default_cacertfile/0","doc":"Returns the default CAStore","ref":"K8s.html#default_cacertfile/0"},{"type":"function","title":"K8s.default_discovery_driver/0","doc":"Returns the _default_ driver for discovery.\n\nEach `K8s.Conn` can have its own driver set. If unset, this value will be used.\n\nDefaults to `K8s.Discovery.Driver.HTTP`","ref":"K8s.html#default_discovery_driver/0"},{"type":"function","title":"Example mix config - K8s.default_discovery_driver/0","doc":"In the example below `dev` and `test` clusters will use the File driver, while `prod` will use the HTTP driver.\n\nCluster names can be atoms or strings, but are internally stored as strings.\n\n```elixir\nuse Mix.Config\n\nconfig :k8s,\n  discovery_driver: K8s.Discovery.Driver.File,\n  discovery_opts: [config: \"test/support/discovery/example.json\"]\n```","ref":"K8s.html#default_discovery_driver/0-example-mix-config"},{"type":"function","title":"K8s.default_discovery_opts/0","doc":"Returns default opts for the discovery driver. This is also configurable per `K8s.Conn`","ref":"K8s.html#default_discovery_opts/0"},{"type":"function","title":"K8s.default_http_provider/0","doc":"Returns the default HTTP Provider","ref":"K8s.html#default_http_provider/0"},{"type":"module","title":"K8s.Client","doc":"Kubernetes API Client.\n\nFunctions return `K8s.Operation`s that represent kubernetes operations.\n\nTo run operations pass them to: `run/2`, or `run/3`\n\nWhen specifying kinds the format should either be in the literal kubernetes kind name (eg `\"ServiceAccount\"`)\nor the downcased version seen in kubectl (eg `\"serviceaccount\"`). A string or atom may be used.","ref":"K8s.Client.html"},{"type":"module","title":"Examples - K8s.Client","doc":"```elixir\n\"Deployment\", \"deployment\", :Deployment, :deployment\n\"ServiceAccount\", \"serviceaccount\", :ServiceAccount, :serviceaccount\n\"HorizontalPodAutoscaler\", \"horizontalpodautoscaler\", :HorizontalPodAutoscaler, :horizontalpodautoscaler\n```\n\n`http_opts` to `K8s.Client.Runner` modules are `K8s.Client.HTTPProvider` HTTP options.","ref":"K8s.Client.html#module-examples"},{"type":"function","title":"K8s.Client.apply/2","doc":"Returns a `PATCH` operation to server-side-apply the given resource.\n\n[K8s Docs](https://kubernetes.io/docs/reference/using-api/server-side-apply/):","ref":"K8s.Client.html#apply/2"},{"type":"function","title":"Examples - K8s.Client.apply/2","doc":"Apply a deployment with management parameteres\n    iex> deployment = K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    ...> K8s.Client.apply(deployment, field_manager: \"my-operator\", force: true)\n    %K8s.Operation{\n      method: :patch,\n      verb: :patch,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"test\", name: \"nginx\"],\n      data: K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\"),\n      query_params: [fieldManager: \"my-operator\", force: true],\n      header_params: [\"Content-Type\": \"application/apply-patch+yaml\"],\n    }","ref":"K8s.Client.html#apply/2-examples"},{"type":"function","title":"K8s.Client.apply/5","doc":"Returns a `PATCH` operation to server-side-apply the given subresource given a resource's details and a subresource map.","ref":"K8s.Client.html#apply/5"},{"type":"function","title":"Examples - K8s.Client.apply/5","doc":"Apply a status to a pod:\n    iex> pod_with_status_subresource = K8s.Resource.from_file!(\"test/support/manifests/nginx-pod.yaml\") |> Map.put(\"status\", %{\"message\" => \"some message\"})\n    ...> K8s.Client.apply(\"v1\", \"pods/status\", [namespace: \"default\", name: \"nginx\"], pod_with_status_subresource, field_manager: \"my-operator\", force: true)\n    %K8s.Operation{\n      method: :patch,\n      verb: :patch,\n      api_version: \"v1\",\n      name: \"pods/status\",\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      data: K8s.Resource.from_file!(\"test/support/manifests/nginx-pod.yaml\") |> Map.put(\"status\", %{\"message\" => \"some message\"}),\n      query_params: [fieldManager: \"my-operator\", force: true],\n      header_params: [\"Content-Type\": \"application/apply-patch+yaml\"]\n    }","ref":"K8s.Client.html#apply/5-examples"},{"type":"function","title":"K8s.Client.async/2","doc":"alias of `K8s.Client.Runner.Async.run/3`","ref":"K8s.Client.html#async/2"},{"type":"function","title":"K8s.Client.async/3","doc":"alias of `K8s.Client.Runner.Async.run/3`","ref":"K8s.Client.html#async/3"},{"type":"function","title":"K8s.Client.connect/4","doc":"Returns a `CONNECT` operation for a pods/exec resource by version, kind/resource type, name, and optionally namespace.\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):","ref":"K8s.Client.html#connect/4"},{"type":"function","title":"Examples - K8s.Client.connect/4","doc":"connect to the nginx deployment in the test namespace:\n    iex> K8s.Client.connect(\"apps/v1\", \"pods/exec\", [namespace: \"test\", name: \"nginx\"], command: \"ls\")\n    %K8s.Operation{\n      method: :post,\n      verb: :connect,\n      api_version: \"apps/v1\",\n      name: \"pods/exec\",\n      path_params: [namespace: \"test\", name: \"nginx\"],\n      query_params: [{:stdin, true}, {:stdout, true}, {:stderr, true}, {:tty, false}, {:command, \"ls\"}]\n    }","ref":"K8s.Client.html#connect/4-examples"},{"type":"function","title":"K8s.Client.create/1","doc":"Returns a `POST` `K8s.Operation` to create the given resource.","ref":"K8s.Client.html#create/1"},{"type":"function","title":"Examples - K8s.Client.create/1","doc":"iex>  deployment = K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    ...> K8s.Client.create(deployment)\n    %K8s.Operation{\n      method: :post,\n      path_params: [namespace: \"test\", name: \"nginx\"],\n      verb: :create,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      data: K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    }","ref":"K8s.Client.html#create/1-examples"},{"type":"function","title":"K8s.Client.create/2","doc":"Returns a `POST` `K8s.Operation` to create the given subresource.\n\nUsed for creating subresources like `Scale` or `Eviction`.","ref":"K8s.Client.html#create/2"},{"type":"function","title":"Examples - K8s.Client.create/2","doc":"Evicting a pod\n    iex> pod = K8s.Resource.from_file!(\"test/support/manifests/nginx-pod.yaml\")\n    ...> eviction = K8s.Resource.from_file!(\"test/support/manifests/eviction-policy.yaml\")\n    ...> K8s.Client.create(pod, eviction)\n    %K8s.Operation{\n      api_version: \"v1\",\n      data: K8s.Resource.from_file!(\"test/support/manifests/eviction-policy.yaml\"),\n      method: :post, name: {\"Pod\", \"Eviction\"},\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      verb: :create\n    }","ref":"K8s.Client.html#create/2-examples"},{"type":"function","title":"K8s.Client.create/4","doc":"Returns a `POST` `K8s.Operation` to create the given subresource.\n\nUsed for creating subresources like `Scale` or `Eviction`.","ref":"K8s.Client.html#create/4"},{"type":"function","title":"Examples - K8s.Client.create/4","doc":"Evicting a pod\n    iex> eviction = K8s.Resource.from_file!(\"test/support/manifests/eviction-policy.yaml\")\n    ...>  K8s.Client.create(\"v1\", \"pods/eviction\", [namespace: \"default\", name: \"nginx\"], eviction)\n    %K8s.Operation{\n      api_version: \"v1\",\n      method: :post,\n      name: \"pods/eviction\",\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      verb: :create,\n      data: K8s.Resource.from_file!(\"test/support/manifests/eviction-policy.yaml\")\n    }","ref":"K8s.Client.html#create/4-examples"},{"type":"function","title":"K8s.Client.delete/1","doc":"Returns a `DELETE` operation for a resource by manifest. May be a partial manifest as long as it contains:\n\n* apiVersion\n* kind\n* metadata.name\n* metadata.namespace (if applicable)\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> Delete will delete a resource. Depending on the specific resource, child objects may or may not be garbage collected by the server. See notes on specific resource objects for details.","ref":"K8s.Client.html#delete/1"},{"type":"function","title":"Examples - K8s.Client.delete/1","doc":"iex> deployment = K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    ...> K8s.Client.delete(deployment)\n    %K8s.Operation{\n      method: :delete,\n      verb: :delete,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"test\", name: \"nginx\"]\n    }","ref":"K8s.Client.html#delete/1-examples"},{"type":"function","title":"K8s.Client.delete/3","doc":"Returns a `DELETE` operation for a resource by version, kind, name, and optionally namespace.","ref":"K8s.Client.html#delete/3"},{"type":"function","title":"Examples - K8s.Client.delete/3","doc":"iex> K8s.Client.delete(\"apps/v1\", \"Deployment\", namespace: \"test\", name: \"nginx\")\n    %K8s.Operation{\n      method: :delete,\n      verb: :delete,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"test\", name: \"nginx\"]\n    }","ref":"K8s.Client.html#delete/3-examples"},{"type":"function","title":"K8s.Client.delete_all/2","doc":"Returns a `DELETE` collection operation for all instances of a cluster scoped resource kind.","ref":"K8s.Client.html#delete_all/2"},{"type":"function","title":"Examples - K8s.Client.delete_all/2","doc":"iex> K8s.Client.delete_all(\"extensions/v1beta1\", \"PodSecurityPolicy\")\n    %K8s.Operation{\n      method: :delete,\n      verb: :deletecollection,\n      api_version: \"extensions/v1beta1\",\n      name: \"PodSecurityPolicy\",\n      path_params: []\n    }\n\n    iex> K8s.Client.delete_all(\"storage.k8s.io/v1\", \"StorageClass\")\n    %K8s.Operation{\n      method: :delete,\n      verb: :deletecollection,\n      api_version: \"storage.k8s.io/v1\",\n      name: \"StorageClass\",\n      path_params: []\n    }","ref":"K8s.Client.html#delete_all/2-examples"},{"type":"function","title":"K8s.Client.delete_all/3","doc":"Returns a `DELETE` collection operation for all instances of a resource kind in a specific namespace.","ref":"K8s.Client.html#delete_all/3"},{"type":"function","title":"Examples - K8s.Client.delete_all/3","doc":"iex> K8s.Client.delete_all(\"apps/v1beta1\", \"ControllerRevision\", namespace: \"default\")\n    %K8s.Operation{\n      method: :delete,\n      verb: :deletecollection,\n      api_version: \"apps/v1beta1\",\n      name: \"ControllerRevision\",\n      path_params: [namespace: \"default\"]\n    }\n\n    iex> K8s.Client.delete_all(\"apps/v1\", \"Deployment\", namespace: \"staging\")\n    %K8s.Operation{\n      method: :delete,\n      verb: :deletecollection,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"staging\"]\n    }","ref":"K8s.Client.html#delete_all/3-examples"},{"type":"function","title":"K8s.Client.get/1","doc":"Returns a `GET` operation for a resource given a Kubernetes manifest. May be a partial manifest as long as it contains:\n\n  * apiVersion\n  * kind\n  * metadata.name\n  * metadata.namespace (if applicable)\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> Get will retrieve a specific resource object by name.","ref":"K8s.Client.html#get/1"},{"type":"function","title":"Examples - K8s.Client.get/1","doc":"Getting a pod\n\n    iex> pod = %{\n    ...>   \"apiVersion\" => \"v1\",\n    ...>   \"kind\" => \"Pod\",\n    ...>   \"metadata\" => %{\"name\" => \"nginx-pod\", \"namespace\" => \"test\"},\n    ...>   \"spec\" => %{\"containers\" => %{\"image\" => \"nginx\"}}\n    ...> }\n    ...> K8s.Client.get(pod)\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      api_version: \"v1\",\n      name: \"Pod\",\n      path_params: [namespace: \"test\", name: \"nginx-pod\"],\n    }","ref":"K8s.Client.html#get/1-examples"},{"type":"function","title":"K8s.Client.get/3","doc":"Returns a `GET` operation for a resource by version, kind/resource type, name, and optionally namespace.\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> Get will retrieve a specific resource object by name.","ref":"K8s.Client.html#get/3"},{"type":"function","title":"Examples - K8s.Client.get/3","doc":"Get the nginx deployment in the default namespace:\n    iex> K8s.Client.get(\"apps/v1\", \"Deployment\", namespace: \"test\", name: \"nginx\")\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"test\", name: \"nginx\"]\n    }\n\n  Get the nginx deployment in the default namespace by passing the kind as atom.\n    iex> K8s.Client.get(\"apps/v1\", :deployment, namespace: \"test\", name: \"nginx\")\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      api_version: \"apps/v1\",\n      name: :deployment,\n      path_params: [namespace: \"test\", name: \"nginx\"]}\n\n  Get the nginx deployment's status:\n    iex> K8s.Client.get(\"apps/v1\", \"deployments/status\", namespace: \"test\", name: \"nginx\")\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      api_version: \"apps/v1\",\n      name: \"deployments/status\",\n      path_params: [namespace: \"test\", name: \"nginx\"]}\n\n  Get the nginx deployment's scale:\n    iex> K8s.Client.get(\"v1\", \"deployments/scale\", namespace: \"test\", name: \"nginx\")\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      api_version: \"v1\",\n      name: \"deployments/scale\",\n      path_params: [namespace: \"test\", name: \"nginx\"]}","ref":"K8s.Client.html#get/3-examples"},{"type":"function","title":"K8s.Client.list/3","doc":"Returns a `GET` operation to list all resources by version, kind, and namespace.\n\nGiven the namespace `:all` as an atom, will perform a list across all namespaces.\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> List will retrieve all resource objects of a specific type within a namespace, and the results can be restricted to resources matching a selector query.\n> List All Namespaces: Like List but retrieves resources across all namespaces.","ref":"K8s.Client.html#list/3"},{"type":"function","title":"Examples - K8s.Client.list/3","doc":"iex> K8s.Client.list(\"v1\", \"Pod\", namespace: \"default\")\n    %K8s.Operation{\n      method: :get,\n      verb: :list,\n      api_version: \"v1\",\n      name: \"Pod\",\n      path_params: [namespace: \"default\"]\n    }\n\n    iex> K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: :all)\n    %K8s.Operation{\n      method: :get,\n      verb: :list_all_namespaces,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: []\n    }","ref":"K8s.Client.html#list/3-examples"},{"type":"function","title":"K8s.Client.parallel/3","doc":"alias of `K8s.Client.Runner.Async.run/3`","ref":"K8s.Client.html#parallel/3"},{"type":"function","title":"K8s.Client.patch/1","doc":"Returns a `PATCH` operation to patch the given resource.\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> Patch will apply a change to a specific field. How the change is merged is defined per field. Lists may either be replaced or merged. Merging lists will not preserve ordering.\n> Patches will never cause optimistic locking failures, and the last write will win. Patches are recommended when the full state is not read before an update, or when failing on optimistic locking is undesirable. When patching complex types, arrays and maps, how the patch is applied is defined on a per-field basis and may either replace the field's current value, or merge the contents into the current value.","ref":"K8s.Client.html#patch/1"},{"type":"function","title":"Examples - K8s.Client.patch/1","doc":"iex>  deployment = K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    ...> K8s.Client.patch(deployment)\n    %K8s.Operation{\n      method: :patch,\n      verb: :patch,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"test\", name: \"nginx\"],\n      data: K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\"),\n      header_params: [\"Content-Type\": \"application/merge-patch+json\"]\n    }","ref":"K8s.Client.html#patch/1-examples"},{"type":"function","title":"K8s.Client.patch/2","doc":"","ref":"K8s.Client.html#patch/2"},{"type":"function","title":"K8s.Client.patch/3","doc":"Returns a `PATCH` operation to patch the given subresource given a resource map and a subresource map.","ref":"K8s.Client.html#patch/3"},{"type":"function","title":"K8s.Client.patch/5","doc":"Returns a `PATCH` operation to patch the given subresource given a resource's details and a subresource map.","ref":"K8s.Client.html#patch/5"},{"type":"function","title":"K8s.Client.put_conn/2","doc":"","ref":"K8s.Client.html#put_conn/2"},{"type":"function","title":"K8s.Client.run/1","doc":"alias of `K8s.Client.Runner.Base.run/1`","ref":"K8s.Client.html#run/1"},{"type":"function","title":"K8s.Client.run/2","doc":"alias of `K8s.Client.Runner.Base.run/2`","ref":"K8s.Client.html#run/2"},{"type":"function","title":"K8s.Client.run/3","doc":"alias of `K8s.Client.Runner.Base.run/3`","ref":"K8s.Client.html#run/3"},{"type":"function","title":"K8s.Client.stream/1","doc":"alias of `K8s.Client.Runner.Stream.run/1`","ref":"K8s.Client.html#stream/1"},{"type":"function","title":"K8s.Client.stream/2","doc":"alias of `K8s.Client.Runner.Stream.run/2`","ref":"K8s.Client.html#stream/2"},{"type":"function","title":"K8s.Client.stream/3","doc":"alias of `K8s.Client.Runner.Stream.run/3`","ref":"K8s.Client.html#stream/3"},{"type":"function","title":"K8s.Client.stream_to/2","doc":"alias of `K8s.Client.Runner.StreamTo.run/2`","ref":"K8s.Client.html#stream_to/2"},{"type":"function","title":"K8s.Client.stream_to/3","doc":"alias of `K8s.Client.Runner.StreamTo.run/3`","ref":"K8s.Client.html#stream_to/3"},{"type":"function","title":"K8s.Client.stream_to/4","doc":"alias of `K8s.Client.Runner.StreamTo.run/4`","ref":"K8s.Client.html#stream_to/4"},{"type":"function","title":"K8s.Client.update/1","doc":"Returns a `PUT` operation to replace/update the given resource.\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> Replacing a resource object will update the resource by replacing the existing spec with the provided one. For read-then-write operations this is safe because an optimistic lock failure will occur if the resource was modified between the read and write. Note: The ResourceStatus will be ignored by the system and will not be updated. To update the status, one must invoke the specific status update operation.\n> Note: Replacing a resource object may not result immediately in changes being propagated to downstream objects. For instance replacing a ConfigMap or Secret resource will not result in all Pods seeing the changes unless the Pods are restarted out of band.","ref":"K8s.Client.html#update/1"},{"type":"function","title":"Examples - K8s.Client.update/1","doc":"iex>  deployment = K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    ...> K8s.Client.update(deployment)\n    %K8s.Operation{\n      method: :put,\n      verb: :update,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: [namespace: \"test\", name: \"nginx\"],\n      data: K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    }","ref":"K8s.Client.html#update/1-examples"},{"type":"function","title":"K8s.Client.update/2","doc":"Returns a `PUT` operation to replace/update the given subresource given a resource map and a subresource map.\n\nUsed for updating subresources like `Scale` or `Status`.","ref":"K8s.Client.html#update/2"},{"type":"function","title":"Examples - K8s.Client.update/2","doc":"Scaling a deployment:\n    iex> deployment = K8s.Resource.from_file!(\"test/support/manifests/nginx-deployment.yaml\")\n    ...> scale = K8s.Resource.from_file!(\"test/support/manifests/scale-replicas.yaml\")\n    ...> K8s.Client.update(deployment, scale)\n    %K8s.Operation{\n      api_version: \"apps/v1\",\n      method: :put,\n      path_params: [namespace: \"test\", name: \"nginx\"],\n      verb: :update,\n      data: K8s.Resource.from_file!(\"test/support/manifests/scale-replicas.yaml\"),\n      name: {\"Deployment\", \"Scale\"}\n    }","ref":"K8s.Client.html#update/2-examples"},{"type":"function","title":"K8s.Client.update/4","doc":"Returns a `PUT` operation to replace/update the given subresource given a resource's details and a subresource map.\n\nUsed for updating subresources like `Scale` or `Status`.","ref":"K8s.Client.html#update/4"},{"type":"function","title":"Examples - K8s.Client.update/4","doc":"Scaling a deployment\n    iex> scale = K8s.Resource.from_file!(\"test/support/manifests/scale-replicas.yaml\")\n    ...>  K8s.Client.update(\"apps/v1\", \"deployments/scale\", [namespace: \"default\", name: \"nginx\"], scale)\n    %K8s.Operation{\n      api_version: \"apps/v1\",\n      data: K8s.Resource.from_file!(\"test/support/manifests/scale-replicas.yaml\"),\n      method: :put,\n      name: \"deployments/scale\",\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      verb: :update\n    }","ref":"K8s.Client.html#update/4-examples"},{"type":"function","title":"K8s.Client.wait_until/2","doc":"alias of `K8s.Client.Runner.Wait.run/2`","ref":"K8s.Client.html#wait_until/2"},{"type":"function","title":"K8s.Client.wait_until/3","doc":"alias of `K8s.Client.Runner.Wait.run/3`","ref":"K8s.Client.html#wait_until/3"},{"type":"function","title":"K8s.Client.watch/3","doc":"Returns a `GET` operation to list all resources by version, kind, and namespace.\n\nGiven the namespace `:all` as an atom, will perform a list across all namespaces.\n\n[K8s Docs](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/):\n\n> List will retrieve all resource objects of a specific type within a namespace, and the results can be restricted to resources matching a selector query.\n> List All Namespaces: Like List but retrieves resources across all namespaces.","ref":"K8s.Client.html#watch/3"},{"type":"function","title":"Examples - K8s.Client.watch/3","doc":"iex> K8s.Client.list(\"v1\", \"Pod\", namespace: \"default\")\n    %K8s.Operation{\n      method: :get,\n      verb: :list,\n      api_version: \"v1\",\n      name: \"Pod\",\n      path_params: [namespace: \"default\"]\n    }\n\n    iex> K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: :all)\n    %K8s.Operation{\n      method: :get,\n      verb: :list_all_namespaces,\n      api_version: \"apps/v1\",\n      name: \"Deployment\",\n      path_params: []\n    }","ref":"K8s.Client.html#watch/3-examples"},{"type":"type","title":"K8s.Client.path_param/0","doc":"","ref":"K8s.Client.html#t:path_param/0"},{"type":"type","title":"K8s.Client.path_params/0","doc":"","ref":"K8s.Client.html#t:path_params/0"},{"type":"module","title":"K8s.Client.DynamicHTTPProvider","doc":"Allows for registration of `K8s.Client.Provider` handlers per-process.\n\nUsed internally by the test suite for testing/mocking Kubernetes responses.","ref":"K8s.Client.DynamicHTTPProvider.html"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"K8s.Client.DynamicHTTPProvider.html#child_spec/1"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.list/0","doc":"List all registered handlers","ref":"K8s.Client.DynamicHTTPProvider.html#list/0"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.locate/1","doc":"Locate the handler module for this process or any ancestor","ref":"K8s.Client.DynamicHTTPProvider.html#locate/1"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.register/2","doc":"Register the handler mdoule for this process","ref":"K8s.Client.DynamicHTTPProvider.html#register/2"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.request/5","doc":"Dispatch `request/5` to the module registered in the current process or any ancestor.","ref":"K8s.Client.DynamicHTTPProvider.html#request/5"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.start_link/1","doc":"Starts this provider.","ref":"K8s.Client.DynamicHTTPProvider.html#start_link/1"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.stream/5","doc":"Dispatch `stream_to/6` to the module registered in the current process or any ancestor.","ref":"K8s.Client.DynamicHTTPProvider.html#stream/5"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.stream_to/6","doc":"Dispatch `stream_to/6` to the module registered in the current process or any ancestor.","ref":"K8s.Client.DynamicHTTPProvider.html#stream_to/6"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.websocket_request/3","doc":"Dispatch `request/5` to the module registered in the current process or any ancestor.","ref":"K8s.Client.DynamicHTTPProvider.html#websocket_request/3"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.websocket_stream/3","doc":"Dispatch `request/5` to the module registered in the current process or any ancestor.","ref":"K8s.Client.DynamicHTTPProvider.html#websocket_stream/3"},{"type":"function","title":"K8s.Client.DynamicHTTPProvider.websocket_stream_to/4","doc":"Dispatch `request/5` to the module registered in the current process or any ancestor.","ref":"K8s.Client.DynamicHTTPProvider.html#websocket_stream_to/4"},{"type":"module","title":"K8s.Client.HTTPStream","doc":"Helper functions for HTTP stream processing.","ref":"K8s.Client.HTTPStream.html"},{"type":"function","title":"K8s.Client.HTTPStream.decode_json_objects/1","doc":"","ref":"K8s.Client.HTTPStream.html#decode_json_objects/1"},{"type":"function","title":"K8s.Client.HTTPStream.transform_to_lines/1","doc":"","ref":"K8s.Client.HTTPStream.html#transform_to_lines/1"},{"type":"type","title":"K8s.Client.HTTPStream.to_lines_t/0","doc":"","ref":"K8s.Client.HTTPStream.html#t:to_lines_t/0"},{"type":"module","title":"K8s.Client.HTTPTestHelper","doc":"HTTP Helpers for test suite.","ref":"K8s.Client.HTTPTestHelper.html"},{"type":"function","title":"K8s.Client.HTTPTestHelper.render/1","doc":"","ref":"K8s.Client.HTTPTestHelper.html#render/1"},{"type":"function","title":"K8s.Client.HTTPTestHelper.render/2","doc":"","ref":"K8s.Client.HTTPTestHelper.html#render/2"},{"type":"function","title":"K8s.Client.HTTPTestHelper.stream_object/1","doc":"","ref":"K8s.Client.HTTPTestHelper.html#stream_object/1"},{"type":"module","title":"K8s.Client.Mint.ConnectionRegistry","doc":"A registry for open connections. As HTTP/2 allows simultaneous requests,\nwe handle multiple requests with one process. In this case, the registry\nstores the PID of the HTTPAdapter which is connected according to the\ngiven connection details (URI/opts).\n\nHTTP/1 only allows one request per connection at a time. In order to\nsupport simultaneous requests, we need a connection pool. This is where\nthe `:poolboy` library comes in.","ref":"K8s.Client.Mint.ConnectionRegistry.html"},{"type":"function","title":"K8s.Client.Mint.ConnectionRegistry.checkin/1","doc":"","ref":"K8s.Client.Mint.ConnectionRegistry.html#checkin/1"},{"type":"function","title":"K8s.Client.Mint.ConnectionRegistry.checkout/1","doc":"ets a `HTTPAdapter` process from the registry.\n\nIf the returned process is an adapter pool, an adapter is checked out from\nthe pool and a map with both PIDs is returned.\n\nIf the returned process is an adapter process, a map with its PID and `pool`\nset to `nil` is returned.","ref":"K8s.Client.Mint.ConnectionRegistry.html#checkout/1"},{"type":"function","title":"K8s.Client.Mint.ConnectionRegistry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"K8s.Client.Mint.ConnectionRegistry.html#child_spec/1"},{"type":"function","title":"K8s.Client.Mint.ConnectionRegistry.start_link/1","doc":"Starts the registry.","ref":"K8s.Client.Mint.ConnectionRegistry.html#start_link/1"},{"type":"type","title":"K8s.Client.Mint.ConnectionRegistry.adapter_pool_t/0","doc":"","ref":"K8s.Client.Mint.ConnectionRegistry.html#t:adapter_pool_t/0"},{"type":"type","title":"K8s.Client.Mint.ConnectionRegistry.adapter_type_t/0","doc":"","ref":"K8s.Client.Mint.ConnectionRegistry.html#t:adapter_type_t/0"},{"type":"type","title":"K8s.Client.Mint.ConnectionRegistry.uriopts/0","doc":"","ref":"K8s.Client.Mint.ConnectionRegistry.html#t:uriopts/0"},{"type":"module","title":"K8s.Client.Mint.HTTPAdapter","doc":"The Mint client implementation. This module handles both, HTTP requests and\nwebsocket connections.","ref":"K8s.Client.Mint.HTTPAdapter.html"},{"type":"module","title":"Processes - K8s.Client.Mint.HTTPAdapter","doc":"The module creates a process per connection to the Kubernetes API server.\nIt supports `HTTP/2` for HTTP requests, but not for websockets. So while\nan open connection can process multiple requests (if the server supports\n`HTTP/2`), it can only process one single websocket connection.\nTherefore, each websocket connection is handled in its own process.","ref":"K8s.Client.Mint.HTTPAdapter.html#module-processes"},{"type":"module","title":"State - K8s.Client.Mint.HTTPAdapter","doc":"The module keeps track of the `Mint.HTTP` connection struct and a map of\npending requests (`K8s.Client.Mint.Request`) for that connection, indexed by the\n`Mint.Types.request_ref()`.\n\n#","ref":"K8s.Client.Mint.HTTPAdapter.html#module-state"},{"type":"module","title":"Requests - K8s.Client.Mint.HTTPAdapter","doc":"Requests are calls to the GenServer that immediately return while the GenServer\nreceives the response parts in the background. The way these response parts are\nreturned depends on the `state_to` argument passed to `request/7` resp.\n`websocket_request/5`. See these function's docs for more details.","ref":"K8s.Client.Mint.HTTPAdapter.html#module-requests"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"K8s.Client.Mint.HTTPAdapter.html#child_spec/1"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.connection_args/2","doc":"Returns connection arguments for the given `URI` and HTTP options.","ref":"K8s.Client.Mint.HTTPAdapter.html#connection_args/2"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.open?/2","doc":"","ref":"K8s.Client.Mint.HTTPAdapter.html#open?/2"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.recv/2","doc":"Returns response parts / frames that were buffered by the process. The\n`stream_to` must have been set to `nil` in `request/7` or\n`websocket_request/5`.\n\nIf the buffer is empty, this call blocks until the next response part /\nframe is received.","ref":"K8s.Client.Mint.HTTPAdapter.html#recv/2"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.request/7","doc":"Starts a HTTP request. The way the response parts are returned depends on the\n`stream_to` argument passed to it.\n\n  - `nil` - response parts are buffered. In order to receive them, the caller\n    needs to `recv/2` passing it the `request_ref` returned by this function.\n  - `pid` - response parts are sent to the process with the given `pid`.\n  - `{pid, reference}` - response parts are sent to the process with the given\n    `pid`. Messages are of the form `{reference, response_part}`.","ref":"K8s.Client.Mint.HTTPAdapter.html#request/7"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.start_link/1","doc":"Opens a connection to Kubernetes, defined by `uri` and `opts`,\nand starts the GenServer.","ref":"K8s.Client.Mint.HTTPAdapter.html#start_link/1"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.websocket_request/5","doc":"Upgrades to a Websocket connection. The way the frames are returned depends\non the `stream_to` argument passed to it.\n\n  - `nil` - frames are buffered. In order to receive them, the caller\n    needs to `recv/2` passing it the `request_ref` returned by this function.\n  - `pid` - frames are sent to the process with the given `pid`.\n  - `{pid, reference}` - frames are sent to the process with the given\n    `pid`. Messages are of the form `{reference, response_part}`.","ref":"K8s.Client.Mint.HTTPAdapter.html#websocket_request/5"},{"type":"function","title":"K8s.Client.Mint.HTTPAdapter.websocket_send/3","doc":"Sends the given `data` throught the websocket specified by the `request_ref`.","ref":"K8s.Client.Mint.HTTPAdapter.html#websocket_send/3"},{"type":"type","title":"K8s.Client.Mint.HTTPAdapter.connection_args_t/0","doc":"","ref":"K8s.Client.Mint.HTTPAdapter.html#t:connection_args_t/0"},{"type":"type","title":"K8s.Client.Mint.HTTPAdapter.t/0","doc":"Describes the state of the connection.\n\n- `:conn` - The current state of the `Mint` connection.\n- `:requests` - The opened requests over this connection (Only `HTTP/2` connections will hold multiple entries in this field.)","ref":"K8s.Client.Mint.HTTPAdapter.html#t:t/0"},{"type":"module","title":"K8s.Client.Mint.Request","doc":"Maintains the state of a HTTP or Websocket request.","ref":"K8s.Client.Mint.Request.html"},{"type":"function","title":"K8s.Client.Mint.Request.map_frame/1","doc":"","ref":"K8s.Client.Mint.Request.html#map_frame/1"},{"type":"function","title":"K8s.Client.Mint.Request.map_outgoing_frame/1","doc":"","ref":"K8s.Client.Mint.Request.html#map_outgoing_frame/1"},{"type":"function","title":"K8s.Client.Mint.Request.map_response/1","doc":"","ref":"K8s.Client.Mint.Request.html#map_response/1"},{"type":"function","title":"K8s.Client.Mint.Request.maybe_terminate_request/1","doc":"","ref":"K8s.Client.Mint.Request.html#maybe_terminate_request/1"},{"type":"function","title":"K8s.Client.Mint.Request.new/1","doc":"","ref":"K8s.Client.Mint.Request.html#new/1"},{"type":"function","title":"K8s.Client.Mint.Request.put_response/2","doc":"","ref":"K8s.Client.Mint.Request.html#put_response/2"},{"type":"function","title":"K8s.Client.Mint.Request.receive_upgrade_response/2","doc":"","ref":"K8s.Client.Mint.Request.html#receive_upgrade_response/2"},{"type":"function","title":"K8s.Client.Mint.Request.recv/2","doc":"","ref":"K8s.Client.Mint.Request.html#recv/2"},{"type":"function","title":"K8s.Client.Mint.Request.stream_request_body/2","doc":"","ref":"K8s.Client.Mint.Request.html#stream_request_body/2"},{"type":"type","title":"K8s.Client.Mint.Request.request_modes/0","doc":"Describes the mode the request is currently in.\n\n- `::pending` - The request is still streaming its body to the server\n- `:receiving` - The request is currently receiving response parts / frames\n- `:closing` - Websocket requests only: The `:close` frame was received but the process wasn't terminated yet\n- `:terminating` - HTTP requests only: The `:done` part was received but the request isn't cleaned up yet","ref":"K8s.Client.Mint.Request.html#t:request_modes/0"},{"type":"type","title":"K8s.Client.Mint.Request.t/0","doc":"Defines the state of the request.\n\n- `:request_ref` - Mint request reference\n- `:caller_ref` - Monitor reference of the calling process.\n- `:stream_to` - the process expecting response parts sent to.\n- `:pool` - the PID of the pool so we can checkin after the last part is sent.\n- `:websocket` - for WebSocket requests: The websocket state (`Mint.WebSocket.t()`).\n- `:mode` - defines what mode the request is currently in.\n- `:buffer` - Holds the buffered response parts or frames that haven't been\n  sent to / received by the caller yet\n- `:pending_request_body` - Part of the request body that has not been sent yet.","ref":"K8s.Client.Mint.Request.html#t:t/0"},{"type":"module","title":"K8s.Client.MintHTTPProvider","doc":"Mint based `K8s.Client.Provider`","ref":"K8s.Client.MintHTTPProvider.html"},{"type":"behaviour","title":"K8s.Client.Provider","doc":"HTTP Request / Response provider behaviour","ref":"K8s.Client.Provider.html"},{"type":"function","title":"K8s.Client.Provider.headers/1","doc":"Generates HTTP headers from `K8s.Conn.RequestOptions`\n\n* Adds `{:Accept, \"application/json\"}` to all requests if the header is not set.","ref":"K8s.Client.Provider.html#headers/1"},{"type":"function","title":"Examples - K8s.Client.Provider.headers/1","doc":"Sets `Content-Type` to `application/json`\n    iex> opts = %K8s.Conn.RequestOptions{headers: [Authorization: \"Basic AF\"]}\n    ...> K8s.Client.HTTPProvider.headers(opts)\n    [Accept: \"application/json\", Authorization: \"Basic AF\"]","ref":"K8s.Client.Provider.html#headers/1-examples"},{"type":"callback","title":"K8s.Client.Provider.request/5","doc":"Perform HTTP requests","ref":"K8s.Client.Provider.html#c:request/5"},{"type":"callback","title":"K8s.Client.Provider.stream/5","doc":"Perform HTTP requests and returns a stream of response chunks","ref":"K8s.Client.Provider.html#c:stream/5"},{"type":"callback","title":"K8s.Client.Provider.stream_to/6","doc":"Perform HTTP requests and stream response to another process","ref":"K8s.Client.Provider.html#c:stream_to/6"},{"type":"callback","title":"K8s.Client.Provider.websocket_request/3","doc":"Perform HTTP requests","ref":"K8s.Client.Provider.html#c:websocket_request/3"},{"type":"callback","title":"K8s.Client.Provider.websocket_stream/3","doc":"Perform websocket requests and returns a stream of response chunks","ref":"K8s.Client.Provider.html#c:websocket_stream/3"},{"type":"callback","title":"K8s.Client.Provider.websocket_stream_to/4","doc":"Perform websocket requests and returns a stream of response chunks","ref":"K8s.Client.Provider.html#c:websocket_stream_to/4"},{"type":"type","title":"K8s.Client.Provider.error_t/0","doc":"","ref":"K8s.Client.Provider.html#t:error_t/0"},{"type":"type","title":"K8s.Client.Provider.headers/0","doc":"","ref":"K8s.Client.Provider.html#t:headers/0"},{"type":"type","title":"K8s.Client.Provider.http_chunk_t/0","doc":"","ref":"K8s.Client.Provider.html#t:http_chunk_t/0"},{"type":"type","title":"K8s.Client.Provider.response_t/0","doc":"","ref":"K8s.Client.Provider.html#t:response_t/0"},{"type":"type","title":"K8s.Client.Provider.send_callback/0","doc":"","ref":"K8s.Client.Provider.html#t:send_callback/0"},{"type":"type","title":"K8s.Client.Provider.stream_response_t/0","doc":"","ref":"K8s.Client.Provider.html#t:stream_response_t/0"},{"type":"type","title":"K8s.Client.Provider.stream_success_t/0","doc":"","ref":"K8s.Client.Provider.html#t:stream_success_t/0"},{"type":"type","title":"K8s.Client.Provider.stream_to_response_t/0","doc":"","ref":"K8s.Client.Provider.html#t:stream_to_response_t/0"},{"type":"type","title":"K8s.Client.Provider.success_t/0","doc":"","ref":"K8s.Client.Provider.html#t:success_t/0"},{"type":"type","title":"K8s.Client.Provider.websocket_chunk_t/0","doc":"","ref":"K8s.Client.Provider.html#t:websocket_chunk_t/0"},{"type":"type","title":"K8s.Client.Provider.websocket_response_t/0","doc":"","ref":"K8s.Client.Provider.html#t:websocket_response_t/0"},{"type":"type","title":"K8s.Client.Provider.websocket_success_t/0","doc":"","ref":"K8s.Client.Provider.html#t:websocket_success_t/0"},{"type":"module","title":"K8s.Client.Runner.Async","doc":"`K8s.Client` runner to process a batch of operations in parallel.","ref":"K8s.Client.Runner.Async.html"},{"type":"function","title":"K8s.Client.Runner.Async.run/3","doc":"Runs multiple operations in parallel. Operations will be returned in same order given.\nOperations will not cease in event of failure.","ref":"K8s.Client.Runner.Async.html#run/3"},{"type":"function","title":"Example - K8s.Client.Runner.Async.run/3","doc":"Get a list of pods in parallel:\n\n  ```elixir\n  pods_to_get = [\n    %{\"name\" => \"nginx\", \"namespace\" => \"default\"},\n    %{\"name\" => \"redis\", \"namespace\" => \"default\"}\n  ]\n\n  # Map each one to an individual `GET` operation.\n  operations = Enum.map(pods_to_get, fn(%{\"name\" => name, \"namespace\" => ns}) ->\n     K8s.Client.get(\"v1\", \"Pod\", namespace: ns, name: name)\n  end)\n\n  # Get the results asynchronously\n  {:ok, conn} = K8s.Conn.from_file(\"test/support/kube-config.yaml\")\n  results = K8s.Client.Runner.Async.run(conn, operations)\n  ```","ref":"K8s.Client.Runner.Async.html#run/3-example"},{"type":"module","title":"K8s.Client.Runner.Base","doc":"Base HTTP processor for `K8s.Client`.","ref":"K8s.Client.Runner.Base.html"},{"type":"function","title":"K8s.Client.Runner.Base.run/1","doc":"Runs a `K8s.Operation`.","ref":"K8s.Client.Runner.Base.html#run/1"},{"type":"function","title":"Examples - K8s.Client.Runner.Base.run/1","doc":"*Note:* Examples assume a `K8s.Conn` was configured named `\"test\"`. See `K8s.Conn.Config`.\n\nRunning a list pods operation:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"test/support/kube-config.yaml\")\noperation = K8s.Client.list(\"v1\", \"Pod\", namespace: :all)\n{:ok, %{\"items\" => pods}} = K8s.Client.run(conn, operation)\n```\n\nRunning a dry-run of a create deployment operation:\n\n```elixir\ndeployment = %{\n  \"apiVersion\" => \"apps/v1\",\n  \"kind\" => \"Deployment\",\n  \"metadata\" => %{\n    \"labels\" => %{\n      \"app\" => \"nginx\"\n    },\n    \"name\" => \"nginx\",\n    \"namespace\" => \"test\"\n  },\n  \"spec\" => %{\n    \"replicas\" => 2,\n    \"selector\" => %{\n      \"matchLabels\" => %{\n        \"app\" => \"nginx\"\n      }\n    },\n    \"template\" => %{\n      \"metadata\" => %{\n        \"labels\" => %{\n          \"app\" => \"nginx\"\n        }\n      },\n      \"spec\" => %{\n        \"containers\" => %{\n          \"image\" => \"nginx\",\n          \"name\" => \"nginx\"\n        }\n      }\n    }\n  }\n}\n\noperation =\n  deployment\n  |> K8s.Client.create()\n  |> K8s.Operation.put_query_param(:dryRun, \"all\")\n\n{:ok, conn} = K8s.Conn.from_file(\"test/support/kube-config.yaml\")\n{:ok, result} = K8s.Client.Runner.Base.run(conn, operation)\n```","ref":"K8s.Client.Runner.Base.html#run/1-examples"},{"type":"function","title":"K8s.Client.Runner.Base.run/2","doc":"","ref":"K8s.Client.Runner.Base.html#run/2"},{"type":"function","title":"K8s.Client.Runner.Base.run/3","doc":"Run a connect operation and pass `websocket_driver_opts` to `K8s.Client.WebSocketProvider`\nSee `run/3`","ref":"K8s.Client.Runner.Base.html#run/3"},{"type":"function","title":"K8s.Client.Runner.Base.stream/3","doc":"Runs a `K8s.Operation` and streams the response.","ref":"K8s.Client.Runner.Base.html#stream/3"},{"type":"function","title":"K8s.Client.Runner.Base.stream_to/4","doc":"Runs a `K8s.Operation` and streams the response.","ref":"K8s.Client.Runner.Base.html#stream_to/4"},{"type":"type","title":"K8s.Client.Runner.Base.body_t/0","doc":"Acceptable HTTP body types","ref":"K8s.Client.Runner.Base.html#t:body_t/0"},{"type":"type","title":"K8s.Client.Runner.Base.error_t/0","doc":"","ref":"K8s.Client.Runner.Base.html#t:error_t/0"},{"type":"type","title":"K8s.Client.Runner.Base.result_t/0","doc":"","ref":"K8s.Client.Runner.Base.html#t:result_t/0"},{"type":"module","title":"K8s.Client.Runner.Stream","doc":"Takes a `K8s.Client.list/3` operation and returns an Elixir [`Stream`](https://hexdocs.pm/elixir/Stream.html) of resources.","ref":"K8s.Client.Runner.Stream.html"},{"type":"function","title":"K8s.Client.Runner.Stream.run/1","doc":"Validates operation type before calling `stream/3`. Only supports verbs: `list_all_namespaces` and `list`.","ref":"K8s.Client.Runner.Stream.html#run/1"},{"type":"function","title":"K8s.Client.Runner.Stream.run/2","doc":"","ref":"K8s.Client.Runner.Stream.html#run/2"},{"type":"function","title":"K8s.Client.Runner.Stream.run/3","doc":"","ref":"K8s.Client.Runner.Stream.html#run/3"},{"type":"module","title":"K8s.Client.Runner.Stream.ListRequest","doc":"`:list` `K8s.Operation` encapsulated with pagination and `K8s.Conn`","ref":"K8s.Client.Runner.Stream.ListRequest.html"},{"type":"function","title":"K8s.Client.Runner.Stream.ListRequest.next_item/1","doc":"","ref":"K8s.Client.Runner.Stream.ListRequest.html#next_item/1"},{"type":"function","title":"K8s.Client.Runner.Stream.ListRequest.stream/3","doc":"","ref":"K8s.Client.Runner.Stream.ListRequest.html#stream/3"},{"type":"type","title":"K8s.Client.Runner.Stream.ListRequest.continue_t/0","doc":"Pagination continue token","ref":"K8s.Client.Runner.Stream.ListRequest.html#t:continue_t/0"},{"type":"type","title":"K8s.Client.Runner.Stream.ListRequest.responses_t/0","doc":"","ref":"K8s.Client.Runner.Stream.ListRequest.html#t:responses_t/0"},{"type":"type","title":"K8s.Client.Runner.Stream.ListRequest.t/0","doc":"List operation as a Stream data type","ref":"K8s.Client.Runner.Stream.ListRequest.html#t:t/0"},{"type":"module","title":"K8s.Client.Runner.Stream.Watch","doc":"Watches a `K8s.Client.list/3` operation and returns an Elixir [`Stream`](https://hexdocs.pm/elixir/Stream.html) of events.","ref":"K8s.Client.Runner.Stream.Watch.html"},{"type":"function","title":"K8s.Client.Runner.Stream.Watch.stream/3","doc":"Watches resources and returns an Elixir Stream of events emmitted by kubernetes.\n\n#","ref":"K8s.Client.Runner.Stream.Watch.html#stream/3"},{"type":"function","title":"Example - K8s.Client.Runner.Stream.Watch.stream/3","doc":"iex> {:ok,conn} = K8s.Conn.from_file(\"~/.kube/config\", [context: \"docker-desktop\"])\n    ...> op = K8s.Client.list(\"v1\", \"Namespace\")\n    ...> K8s.Client.Runner.Watch.Stream.stream(conn, op, []) |> Stream.map(&IO.inspect/1) |> Stream.run()","ref":"K8s.Client.Runner.Stream.Watch.html#stream/3-example"},{"type":"type","title":"K8s.Client.Runner.Stream.Watch.t/0","doc":"","ref":"K8s.Client.Runner.Stream.Watch.html#t:t/0"},{"type":"module","title":"K8s.Client.Runner.StreamTo","doc":"Takes a `K8s.Client.list/3` operation and returns an Elixir [`Stream`](https://hexdocs.pm/elixir/Stream.html) of resources.","ref":"K8s.Client.Runner.StreamTo.html"},{"type":"function","title":"K8s.Client.Runner.StreamTo.run/2","doc":"Validates operation type before calling `stream/3`. Only supports verbs: `list_all_namespaces` and `list`.","ref":"K8s.Client.Runner.StreamTo.html#run/2"},{"type":"function","title":"K8s.Client.Runner.StreamTo.run/3","doc":"","ref":"K8s.Client.Runner.StreamTo.html#run/3"},{"type":"function","title":"K8s.Client.Runner.StreamTo.run/4","doc":"","ref":"K8s.Client.Runner.StreamTo.html#run/4"},{"type":"module","title":"K8s.Client.Runner.Wait","doc":"Waiting functionality for `K8s.Client`.\n\nNote: This is built using repeated GET operations rather than using a [watch](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/#watch-list-deployment-v1-apps) operation w/ `fieldSelector`.","ref":"K8s.Client.Runner.Wait.html"},{"type":"function","title":"K8s.Client.Runner.Wait.run/2","doc":"Continually perform a GET based operation until a condition is met.","ref":"K8s.Client.Runner.Wait.html#run/2"},{"type":"function","title":"Example - K8s.Client.Runner.Wait.run/2","doc":"This follow example will wait 60 seconds for the field `status.succeeded` to equal `1`.\n\n```elixir\nop = K8s.Client.get(\"batch/v1\", :job, namespace: \"default\", name: \"sleep\")\nopts = [find: [\"status\", \"succeeded\"], eval: 1, timeout: 60]\n{:ok, conn} = K8s.Conn.from_file(\"test/support/kube-config.yaml\")\nresp = K8s.Client.Runner.Wait.run(conn, op, opts)\n```","ref":"K8s.Client.Runner.Wait.html#run/2-example"},{"type":"function","title":"K8s.Client.Runner.Wait.run/3","doc":"","ref":"K8s.Client.Runner.Wait.html#run/3"},{"type":"type","title":"K8s.Client.Runner.Wait.t/0","doc":"A wait configuration","ref":"K8s.Client.Runner.Wait.html#t:t/0"},{"type":"module","title":"K8s.Conn","doc":"Handles authentication and connection configuration details for a Kubernetes\ncluster. The `%K8s.Conn{}` struct is required in order to run any object\nagainst the cluster. Use any of the functions defined in this module to create\na `%K8s.Conn{}` struct and pass it to the functions of `K8s.Client`.","ref":"K8s.Conn.html"},{"type":"module","title":"Example - K8s.Conn","doc":"```\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\n\n{:ok, default_ns} =\n  K8s.Client.get(\"v1\", \"Namespace\", name: \"default\")\n  |> K8s.Client.put_conn(conn)\n  |> K8s.Client.run()\n```\n\nAlternatively, you can pass `conn` to `K8s.Client.run()`.\n\n```\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\nop = K8s.Client.get(\"v1\", \"Namespace\", name: \"default\")\n{:ok, default_ns} = K8s.Client.run(op, conn)\n```","ref":"K8s.Conn.html#module-example"},{"type":"module","title":"Scenarios - K8s.Conn","doc":"* If your cluster connection is defined in a file, e.g. `~/.kube/config`, use\n  `K8s.Conn.from_file/2`.\n* If running in a pod inside the cluster you're connecting to, use\n  `K8s.Conn.from_service_account/2`\n* If an environment variable points to a config file, use\n  `K8s.Conn.from_env/2`","ref":"K8s.Conn.html#module-scenarios"},{"type":"function","title":"K8s.Conn.from_env/2","doc":"Generates the configuration from a file whose location is defined by the\ngiven `env_var`. Defaults to `KUBECONFIG`.\n\n### Options\n\nSee `from_file/2`.\n\n### Examples\n\nif `KUBECONFIG` is set:\n\n```\n{:ok, conn} = K8s.Conn.from_env()\n```\n\nPass the env variable name:\n\n```\n{:ok, conn} = K8s.Conn.from_env(\"TEST_KUBECONFIG\")\n```\n\nPass the env variable name and options:\n\n```\n{:ok, conn} = K8s.Conn.from_env(\"TEST_KUBECONFIG\", insecure_skip_tls_verify: true)\n```","ref":"K8s.Conn.html#from_env/2"},{"type":"function","title":"K8s.Conn.from_file/2","doc":"Reads configuration details from a kubernetes config file.\n\nIf you run your code on your machine, you most likely have a config file at\n`~/.kube/config`. If you created a local cluster using `kind`, `k3d` or\nsimilar, a context entry is either added to that config file or you saved it\nto a specific location upon cluster creation. Either way, this function reads\nthe config from any of these files.\n\n#","ref":"K8s.Conn.html#from_file/2"},{"type":"function","title":"Example - K8s.Conn.from_file/2","doc":"Using the currently selected context:\n\n```\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\n```\n\nPass the context and allow insecure TLS verification :\n\n```\n{:ok, conn} =\n  K8s.Conn.from_file(\"~/.kube/config\",\n    context: \"my-kind-cluster\",\n    insecure_skip_tls_verify: true\n  )\n```\n\n#","ref":"K8s.Conn.html#from_file/2-example"},{"type":"function","title":"Options - K8s.Conn.from_file/2","doc":"* `:context` - sets an alternate context - defaults to `current-context`.\n* `:cluster` - set or override the cluster read from the context\n* `:user`-  set or override the user read from the context\n* `:discovery_driver` - module name to use for discovery\n* `:discovery_opts` - options for discovery module\n* `:insecure_skip_tls_verify` - Skip TLS verification","ref":"K8s.Conn.html#from_file/2-options"},{"type":"function","title":"K8s.Conn.from_service_account/2","doc":"Generates the configuration from a Kubernetes service account.\n\nThis is used when running in a Pod inside the cluster you're accessing. Make\nsure to setup RBAC for the service account running the Pod.\n\nDocumentation: [kubernetes.io :: Accessing the API from a\nPod](https://kubernetes.io/docs/tasks/access-application-cluster/access-cluster/#accessing-the-api-from-a-pod)\n\n#","ref":"K8s.Conn.html#from_service_account/2"},{"type":"function","title":"Options - K8s.Conn.from_service_account/2","doc":"* `:insecure_skip_tls_verify` - Skip TLS verification\n\n#","ref":"K8s.Conn.html#from_service_account/2-options"},{"type":"function","title":"Example - K8s.Conn.from_service_account/2","doc":"Using the currently selected context:\n\n```\n{:ok, conn} = K8s.Conn.from_service_account()\n```\n\nYou can set a specific path to the service account token file:\n\n```\n{:ok, conn} =\n  K8s.Conn.from_service_account(\"/path/to/token\",\n    insecure_skip_tls_verify: true\n  )\n```\n\nAllow insecure TLS verification:\n\n```\n{:ok, conn} =\n  K8s.Conn.from_service_account(\n    insecure_skip_tls_verify: true\n  )\n```\n\n```\n{:ok, conn} =\n  K8s.Conn.from_service_account(\n    \"/path/to/token\",\n    insecure_skip_tls_verify: true\n  )\n```","ref":"K8s.Conn.html#from_service_account/2-example"},{"type":"type","title":"K8s.Conn.t/0","doc":"* `cluster_name` - The cluster name if read from a kubeconfig file\n* `user_name` - The user name if read from a kubeconfig file\n* `namespace` - The namespace if read from a service account token\n* `url` - The Kubernetes API URL","ref":"K8s.Conn.html#t:t/0"},{"type":"behaviour","title":"K8s.Conn.Auth","doc":"Authorization behaviour","ref":"K8s.Conn.Auth.html"},{"type":"callback","title":"K8s.Conn.Auth.create/2","doc":"Creates a struct holding the information to authenticate against the\nAPI server.\n\n#","ref":"K8s.Conn.Auth.html#c:create/2"},{"type":"callback","title":"Arguments - K8s.Conn.Auth.create/2","doc":"* `auth_map` - A map representing the `.users[].user` part from a kubeconfig.\n* `base_path` - The path to the folder holding the kubeconfig file.\n\n#","ref":"K8s.Conn.Auth.html#c:create/2-arguments"},{"type":"callback","title":"Result - K8s.Conn.Auth.create/2","doc":"The struct returned by this function MUST implement the\n`K8s.Conn.RequestOptions` protocol.\nReturn `:skip` to signal to `K8s.Conn` to skip a provider that would not\nauthenticate the current connection.\n\n#","ref":"K8s.Conn.Auth.html#c:create/2-result"},{"type":"callback","title":"Examples - K8s.Conn.Auth.create/2","doc":"See the various modules implementing this behaviour for examples.","ref":"K8s.Conn.Auth.html#c:create/2-examples"},{"type":"module","title":"K8s.Conn.Auth.AuthProvider","doc":"`auth-provider` authentication support","ref":"K8s.Conn.Auth.AuthProvider.html"},{"type":"function","title":"K8s.Conn.Auth.AuthProvider.generate_token/1","doc":"\"Generate\" a token using the `auth-provider` config in kube config.","ref":"K8s.Conn.Auth.AuthProvider.html#generate_token/1"},{"type":"type","title":"K8s.Conn.Auth.AuthProvider.t/0","doc":"","ref":"K8s.Conn.Auth.AuthProvider.html#t:t/0"},{"type":"module","title":"K8s.Conn.Auth.Azure","doc":"`auth-provider` for azure","ref":"K8s.Conn.Auth.Azure.html"},{"type":"function","title":"K8s.Conn.Auth.Azure.refresh_token/4","doc":"","ref":"K8s.Conn.Auth.Azure.html#refresh_token/4"},{"type":"type","title":"K8s.Conn.Auth.Azure.t/0","doc":"","ref":"K8s.Conn.Auth.Azure.html#t:t/0"},{"type":"module","title":"K8s.Conn.Auth.BasicAuth","doc":"basic auth cluster authentication","ref":"K8s.Conn.Auth.BasicAuth.html"},{"type":"type","title":"K8s.Conn.Auth.BasicAuth.t/0","doc":"","ref":"K8s.Conn.Auth.BasicAuth.html#t:t/0"},{"type":"module","title":"K8s.Conn.Auth.Certificate","doc":"Certificate based cluster authentication.","ref":"K8s.Conn.Auth.Certificate.html"},{"type":"type","title":"K8s.Conn.Auth.Certificate.t/0","doc":"","ref":"K8s.Conn.Auth.Certificate.html#t:t/0"},{"type":"module","title":"K8s.Conn.Auth.DigitalOcean","doc":"Cluster authentication for kube configs created by doctl, the DigitalOcean CLI","ref":"K8s.Conn.Auth.DigitalOcean.html"},{"type":"module","title":"K8s.Conn.Auth.Exec","doc":"Cluster authentication for kube configs using an `exec` section.\n\nUseful for Kubernetes clusters running on AWS which use IAM authentication (eg. the `aws-iam-authenticator` binary).\nAn applicable kube config may look something like this:\n\n```\n# ...\nusers:\n- name: staging-user\n  user:\n    exec:\n      # API version to use when decoding the ExecCredentials resource. Required.\n      apiVersion: client.authentication.k8s.io/v1alpha1\n\n      # Command to execute. Required.\n      command: aws-iam-authenticator\n\n      # Arguments to pass when executing the plugin. Optional.\n      args:\n      - token\n      - -i\n      - staging\n\n      # Environment variables to set when executing the plugin. Optional.\n      env:\n      - name: \"FOO\"\n        value: \"bar\"\n```","ref":"K8s.Conn.Auth.Exec.html"},{"type":"function","title":"K8s.Conn.Auth.Exec.generate_token/1","doc":"\"Generate\" a token using the `exec` config in kube config.","ref":"K8s.Conn.Auth.Exec.html#generate_token/1"},{"type":"type","title":"K8s.Conn.Auth.Exec.t/0","doc":"","ref":"K8s.Conn.Auth.Exec.html#t:t/0"},{"type":"module","title":"K8s.Conn.Auth.Token","doc":"Token based cluster authentication","ref":"K8s.Conn.Auth.Token.html"},{"type":"type","title":"K8s.Conn.Auth.Token.t/0","doc":"","ref":"K8s.Conn.Auth.Token.html#t:t/0"},{"type":"module","title":"K8s.Conn.PKI","doc":"Retrieves information from certificates","ref":"K8s.Conn.PKI.html"},{"type":"function","title":"K8s.Conn.PKI.cert_from_base64/1","doc":"Decodes the certificate from a base64 encoded string","ref":"K8s.Conn.PKI.html#cert_from_base64/1"},{"type":"function","title":"K8s.Conn.PKI.cert_from_map/2","doc":"Reads the certificate from PEM file or base64 encoding","ref":"K8s.Conn.PKI.html#cert_from_map/2"},{"type":"function","title":"K8s.Conn.PKI.cert_from_pem/1","doc":"Reads the certificate from a PEM file","ref":"K8s.Conn.PKI.html#cert_from_pem/1"},{"type":"function","title":"K8s.Conn.PKI.private_key_from_base64/1","doc":"Decodes private key from a base64 encoded string","ref":"K8s.Conn.PKI.html#private_key_from_base64/1"},{"type":"function","title":"K8s.Conn.PKI.private_key_from_pem/1","doc":"Reads private key from a PEM file","ref":"K8s.Conn.PKI.html#private_key_from_pem/1"},{"type":"type","title":"K8s.Conn.PKI.private_key_data_t/0","doc":"","ref":"K8s.Conn.PKI.html#t:private_key_data_t/0"},{"type":"protocol","title":"K8s.Conn.RequestOptions","doc":"Encapsulates HTTP request options for an authentication provider.","ref":"K8s.Conn.RequestOptions.html"},{"type":"function","title":"K8s.Conn.RequestOptions.generate/1","doc":"","ref":"K8s.Conn.RequestOptions.html#generate/1"},{"type":"type","title":"K8s.Conn.RequestOptions.generate_t/0","doc":"`generate/1` response type","ref":"K8s.Conn.RequestOptions.html#t:generate_t/0"},{"type":"type","title":"K8s.Conn.RequestOptions.t/0","doc":"HTTP Request options","ref":"K8s.Conn.RequestOptions.html#t:t/0"},{"type":"module","title":"K8s.Discovery","doc":"Kubernetes API Discovery","ref":"K8s.Discovery.html"},{"type":"function","title":"K8s.Discovery.default_driver/0","doc":"","ref":"K8s.Discovery.html#default_driver/0"},{"type":"function","title":"K8s.Discovery.default_opts/0","doc":"","ref":"K8s.Discovery.html#default_opts/0"},{"type":"function","title":"K8s.Discovery.resources/3","doc":"Kubernetes API Resources supported by the cluster.","ref":"K8s.Discovery.html#resources/3"},{"type":"function","title":"K8s.Discovery.url_for/2","doc":"Discovery the URL for a `K8s.Conn` and `K8s.Operation`","ref":"K8s.Discovery.html#url_for/2"},{"type":"function","title":"Examples - K8s.Discovery.url_for/2","doc":"iex> {:ok, conn} = K8s.Conn.from_file(\"./test/support/kube-config.yaml\")\n    ...> op = K8s.Operation.build(:get, \"apps/v1\", :deployments, [namespace: \"default\", name: \"nginx\"])\n    ...> K8s.Discovery.url_for(conn, op)\n    {:ok, \"https://localhost:6443/apis/apps/v1/namespaces/default/deployments/nginx\"}","ref":"K8s.Discovery.html#url_for/2-examples"},{"type":"function","title":"K8s.Discovery.versions/2","doc":"Kubernetes API Versions supported by the cluster.","ref":"K8s.Discovery.html#versions/2"},{"type":"behaviour","title":"K8s.Discovery.Driver","doc":"Driver behaviour for `K8s.Discovery`","ref":"K8s.Discovery.Driver.html"},{"type":"callback","title":"K8s.Discovery.Driver.resources/2","doc":"List of Kubernetes `APIResourceList`s","ref":"K8s.Discovery.Driver.html#c:resources/2"},{"type":"callback","title":"Examples - K8s.Discovery.Driver.resources/2","doc":"iex> {:ok, conn} = K8s.Conn.from_file(\"test/support/kube-config.yaml\")\n    ...> K8s.Discovery.Driver.HTTP.resources(\"autoscaling/v1\", conn)\n    {:ok, [\n             %{\n               \"kind\" => \"DaemonSet\",\n               \"name\" => \"daemonsets\"\n             },\n             %{\n               \"kind\" => \"Deployment\",\n               \"name\" => \"deployments\"\n             },\n             %{\n               \"kind\" => \"Deployment\",\n               \"name\" => \"deployments/status\"\n             }\n           ]}","ref":"K8s.Discovery.Driver.html#c:resources/2-examples"},{"type":"callback","title":"K8s.Discovery.Driver.resources/3","doc":"","ref":"K8s.Discovery.Driver.html#c:resources/3"},{"type":"callback","title":"K8s.Discovery.Driver.versions/1","doc":"List of Kubernetes `apiVersion`s","ref":"K8s.Discovery.Driver.html#c:versions/1"},{"type":"callback","title":"Examples - K8s.Discovery.Driver.versions/1","doc":"iex> {:ok, conn} = K8s.Conn.from_file(\"test/support/kube-config.yaml\")\n    ...> K8s.Discovery.Driver.HTTP.api(conn)\n    {:ok, [\"v1\"]}","ref":"K8s.Discovery.Driver.html#c:versions/1-examples"},{"type":"callback","title":"K8s.Discovery.Driver.versions/2","doc":"","ref":"K8s.Discovery.Driver.html#c:versions/2"},{"type":"type","title":"K8s.Discovery.Driver.driver_error_t/0","doc":"Errors returned by adapters should return an error tuple with an `atom()` describing the error or alternatively an \"error\" struct with more details.","ref":"K8s.Discovery.Driver.html#t:driver_error_t/0"},{"type":"type","title":"Examples - K8s.Discovery.Driver.driver_error_t/0","doc":"```elixir\n{:error, :file_not_found}\n```\n\n```elixir\n{:error, %K8s.Discovery.Driver.MyDriver.FileNotFoundError{config: \"path-to-file\"}}\n```","ref":"K8s.Discovery.Driver.html#t:driver_error_t/0-examples"},{"type":"module","title":"K8s.Discovery.Driver.File","doc":"File Driver for Kubernetes API discovery.\n\nThis module allows API versions and Resources accessible by the k8s to be defined rather than discovered.\n\nThis module is primarily used for testing, but can be used if you wish to hard code supported API versions and resources.\n\nSee [./test/support/discovery/example.json](./test/support/discovery/example.json) for an example.","ref":"K8s.Discovery.Driver.File.html"},{"type":"module","title":"Examples - K8s.Discovery.Driver.File","doc":"\"Discovering\" hard coded resources.\n\n    iex> K8s.Discovery.Driver.File.resources(\"apps/v1\", %K8s.Conn{}, config: \"./test/support/discovery/example.json\")\n    [\n      %{\n        \"kind\" => \"DaemonSet\",\n        \"name\" => \"daemonsets\"\n      },\n      %{\n        \"kind\" => \"Deployment\",\n        \"name\" => \"deployments\"\n      },\n      %{\n        \"kind\" => \"Deployment\",\n        \"name\" => \"deployments/status\"\n      }\n    ]\n\n  \"Discovering\" hard coded versions.\n    iex> K8s.Discovery.Driver.File.versions(%K8s.Conn{}, config: \"./test/support/discovery/example.json\")\n    [\"v1\", \"apps/v1\"]","ref":"K8s.Discovery.Driver.File.html#module-examples"},{"type":"module","title":"K8s.Discovery.Driver.HTTP","doc":"HTTP Driver for Kubernetes API discovery","ref":"K8s.Discovery.Driver.HTTP.html"},{"type":"module","title":"K8s.Discovery.ResourceFinder","doc":"Kubernetes API Groups","ref":"K8s.Discovery.ResourceFinder.html"},{"type":"function","title":"K8s.Discovery.ResourceFinder.find_resource/3","doc":"Finds a resource definition by api version and (name or kind).","ref":"K8s.Discovery.ResourceFinder.html#find_resource/3"},{"type":"function","title":"K8s.Discovery.ResourceFinder.resource_name_for_kind/3","doc":"Get the REST resource name for a kubernetes `Kind`.\n\nSince `K8s.Operation` is abstracted away from a specific cluster, when working with kubernetes resource `Map`s and specifying `\"kind\"` the `K8s.Operation.Path` module isn't\nable to determine the correct path. (It will generate things like /api/v1/Pod instead of api/v1/pods).\n\nAlso accepts REST resource name in the event they are provided, as it may be known in the event of subresources.","ref":"K8s.Discovery.ResourceFinder.html#resource_name_for_kind/3"},{"type":"type","title":"K8s.Discovery.ResourceFinder.error_t/0","doc":"","ref":"K8s.Discovery.ResourceFinder.html#t:error_t/0"},{"type":"module","title":"K8s.Middleware","doc":"Interface for interacting with cluster middleware\n\nWhile middlewares are applied to `K8s.Middleware.Request`s they are registerd by a _cluster name_ which is an atom.\n\nThis allows multiple connections to be used (different credentials) with the same middleware stack.\n\nAny cluster name that has _not_ been initialized, will automatically have the default stack applied.","ref":"K8s.Middleware.html"},{"type":"module","title":"Examples - K8s.Middleware","doc":"Using default middleware\n    iex> conn = %K8s.Conn{}\n    ...> req = %K8s.Middlware.Request{}\n    ...> K8s.Middleware.run(req)\n\n  Adding middlware to a cluster\n    iex> conn = %K8s.Conn{cluster_name: \"foo\"}\n    ...> K8s.Middleware.add(\"foo\", :request, MyMiddlewareModule)\n    ...> req = %K8s.Middlware.Request{}\n    ...> K8s.Middleware.run(req)\n\n  Setting/Replacing middleware on a cluster\n    iex> conn = %K8s.Conn{cluster_name: \"foo\"}\n    ...> K8s.Middleware.set(\"foo\", :request, [MyMiddlewareModule, OtherModule])\n    ...> req = %K8s.Middlware.Request{}\n    ...> K8s.Middleware.run(req)","ref":"K8s.Middleware.html#module-examples"},{"type":"function","title":"K8s.Middleware.run/2","doc":"","ref":"K8s.Middleware.html#run/2"},{"type":"module","title":"K8s.Middleware.Error","doc":"Encapsulates middleware process errors","ref":"K8s.Middleware.Error.html"},{"type":"type","title":"K8s.Middleware.Error.t/0","doc":"Middleware processing error\n\n* `middleware` middleware module that caused the error\n* `request` `K8s.Middleware.Request`\n* `error` actual error, can be `any()` type","ref":"K8s.Middleware.Error.html#t:t/0"},{"type":"behaviour","title":"K8s.Middleware.Request","doc":"HTTP Request middleware","ref":"K8s.Middleware.Request.html"},{"type":"callback","title":"K8s.Middleware.Request.call/1","doc":"Request middleware callback","ref":"K8s.Middleware.Request.html#c:call/1"},{"type":"type","title":"K8s.Middleware.Request.t/0","doc":"Middleware Request type","ref":"K8s.Middleware.Request.html#t:t/0"},{"type":"module","title":"K8s.Middleware.Request.EncodeBody","doc":"Naive JSON body encoder.\n\nEncodes JSON payloads when given an modifiying HTTP verb, otherwise returns an empty string.","ref":"K8s.Middleware.Request.EncodeBody.html"},{"type":"module","title":"K8s.Middleware.Request.Initialize","doc":"Initializes a request with connection details (header and HTTPoison opts) from `K8s.Conn.RequestOptions`","ref":"K8s.Middleware.Request.Initialize.html"},{"type":"module","title":"K8s.Middleware.Stack","doc":"`K8s.Middlware` stacks to apply to a `K8s.Conn`","ref":"K8s.Middleware.Stack.html"},{"type":"function","title":"K8s.Middleware.Stack.default/0","doc":"The default middleware stack","ref":"K8s.Middleware.Stack.html#default/0"},{"type":"type","title":"K8s.Middleware.Stack.t/0","doc":"","ref":"K8s.Middleware.Stack.html#t:t/0"},{"type":"module","title":"K8s.Operation","doc":"Encapsulates Kubernetes REST API operations.","ref":"K8s.Operation.html"},{"type":"function","title":"K8s.Operation.build/3","doc":"Builds an `Operation` given a verb and a k8s resource.","ref":"K8s.Operation.html#build/3"},{"type":"function","title":"Examples - K8s.Operation.build/3","doc":"iex> deploy = %{\"apiVersion\" => \"apps/v1\", \"kind\" => \"Deployment\", \"metadata\" => %{\"namespace\" => \"default\", \"name\" => \"nginx\"}}\n    ...> K8s.Operation.build(:update, deploy)\n    %K8s.Operation{\n      method: :put,\n      verb: :update,\n      data: %{\"apiVersion\" => \"apps/v1\", \"kind\" => \"Deployment\", \"metadata\" => %{\"namespace\" => \"default\", \"name\" => \"nginx\"}},\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      api_version: \"apps/v1\",\n      name: \"Deployment\"\n    }","ref":"K8s.Operation.html#build/3-examples"},{"type":"function","title":"K8s.Operation.build/6","doc":"Builds an `Operation` given an verb and a k8s resource info.\n\n*Note:* The `name` here may be a `Kind` and not a REST resource name in the event that the operation was built using a map.\nUse `K8s.Discovery.ResourceFinder.resource_name_for_kind/3` to get the correct REST resource name, given a `kind`.","ref":"K8s.Operation.html#build/6"},{"type":"function","title":"Examples - K8s.Operation.build/6","doc":"Building a GET deployment operation:\n    iex> K8s.Operation.build(:get, \"apps/v1\", :deployment, [namespace: \"default\", name: \"nginx\"])\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      data: nil,\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      api_version: \"apps/v1\",\n      name: :deployment\n    }\n\nBuilding a GET deployments/status operation:\n    iex> K8s.Operation.build(:get, \"apps/v1\", \"deployments/status\", [namespace: \"default\", name: \"nginx\"])\n    %K8s.Operation{\n      method: :get,\n      verb: :get,\n      data: nil,\n      path_params: [namespace: \"default\", name: \"nginx\"],\n      api_version: \"apps/v1\",\n      name: \"deployments/status\"\n    }","ref":"K8s.Operation.html#build/6-examples"},{"type":"function","title":"K8s.Operation.get_label_selector/1","doc":"","ref":"K8s.Operation.html#get_label_selector/1"},{"type":"function","title":"K8s.Operation.get_query_param/2","doc":"Get a query param of an operation","ref":"K8s.Operation.html#get_query_param/2"},{"type":"function","title":"Examples - K8s.Operation.get_query_param/2","doc":"Using a `keyword` list of params:\n    iex> operation = %K8s.Operation{query_params: [foo: \"bar\"]}\n    ...> K8s.Operation.get_query_param(operation, :foo)\n    \"bar\"","ref":"K8s.Operation.html#get_query_param/2-examples"},{"type":"function","title":"K8s.Operation.get_selector/1","doc":"Gets a `K8s.Selector` on the operation.","ref":"K8s.Operation.html#get_selector/1"},{"type":"function","title":"Examples - K8s.Operation.get_selector/1","doc":"iex> operation = %K8s.Operation{query_params: [labelSelector: K8s.Selector.label({\"component\", \"redis\"})]}\n    ...> K8s.Operation.get_selector(operation)\n    %K8s.Selector{\n      match_expressions: [],\n      match_labels: %{\"component\" => {\"=\", \"redis\"}}\n    }","ref":"K8s.Operation.html#get_selector/1-examples"},{"type":"function","title":"K8s.Operation.put_conn/2","doc":"Set the connection object on the operation","ref":"K8s.Operation.html#put_conn/2"},{"type":"function","title":"Examples - K8s.Operation.put_conn/2","doc":"iex> operation = %K8s.Operation{query_params: [foo: \"bar\"]}\n    ...> conn = %K8s.Conn{}\n    ...> operation = K8s.Operation.put_conn(operation, conn)\n    ...> match?(%K8s.Operation{conn: %K8s.Conn{}}, operation)\n    true","ref":"K8s.Operation.html#put_conn/2-examples"},{"type":"function","title":"K8s.Operation.put_label_selector/2","doc":"","ref":"K8s.Operation.html#put_label_selector/2"},{"type":"function","title":"K8s.Operation.put_query_param/2","doc":"","ref":"K8s.Operation.html#put_query_param/2"},{"type":"function","title":"K8s.Operation.put_query_param/3","doc":"Add a query param to an operation","ref":"K8s.Operation.html#put_query_param/3"},{"type":"function","title":"Examples - K8s.Operation.put_query_param/3","doc":"Using a `keyword` list of params:\n    iex> operation = %K8s.Operation{}\n    ...> K8s.Operation.put_query_param(operation, :foo, \"bar\")\n    %K8s.Operation{query_params: [foo: \"bar\"]}","ref":"K8s.Operation.html#put_query_param/3-examples"},{"type":"function","title":"K8s.Operation.put_selector/2","doc":"Puts a `K8s.Selector` on the operation.","ref":"K8s.Operation.html#put_selector/2"},{"type":"function","title":"Examples - K8s.Operation.put_selector/2","doc":"iex> operation = %K8s.Operation{}\n    ...> selector = K8s.Selector.label({\"component\", \"redis\"})\n    ...> K8s.Operation.put_selector(operation, selector)\n    %K8s.Operation{\n      query_params: [\n        labelSelector: %K8s.Selector{\n          match_expressions: [],\n          match_labels: %{\"component\" => {\"=\", \"redis\"}}\n        }\n      ]\n    }","ref":"K8s.Operation.html#put_selector/2-examples"},{"type":"function","title":"K8s.Operation.to_path/1","doc":"Converts a `K8s.Operation` into a URL path.","ref":"K8s.Operation.html#to_path/1"},{"type":"type","title":"K8s.Operation.name_t/0","doc":"`K8s.Operation` name. May be an atom, string, or tuple of `{resource, subresource}`.","ref":"K8s.Operation.html#t:name_t/0"},{"type":"type","title":"K8s.Operation.patch_type/0","doc":"Acceptable patch types","ref":"K8s.Operation.html#t:patch_type/0"},{"type":"type","title":"K8s.Operation.t/0","doc":"* `api_version` - API `groupVersion`, AKA `apiVersion`\n* `name` - The name of the REST operation (Kubernets kind/resource/subresource). This is *not* _always_ the same as the `kind` key in the `data` field. e.g: `deployments` when POSTing, GETting a deployment.\n* `data` - HTTP request body to submit when applicable. (POST, PUT, PATCH, etc)\n* `method` - HTTP Method\n* `verb` - Kubernetes [REST API verb](https://kubernetes.io/docs/reference/access-authn-authz/authorization/#determine-the-request-verb) (`deletecollection`, `update`, `create`, `watch`, etc)\n* `path_params` - Parameters to interpolate into the Kubernetes REST URL\n* `query_params` - Query parameters. Merged w/ params provided to any `K8s.Client.Runner`. `K8s.Client.Runner` options win.\n* `header_params` - Header parameters.\n\n`name` would be `deployments` in the case of a deployment, but may be `deployments/status` or `deployments/scale` for Status and Scale subresources.\n\n## `name` and `data` field examples\n\nThe following example would `update` the *nginx* deployment's `Scale`. Note the `deployments/scale` operation will have a `Scale` *data* payload:\n\n```elixir\n%K8s.Operation{\n  method: :put,\n  verb: :update,\n  api_version: \"v1\", # api version of the \"Scale\" kind\n  name: \"deployments/scale\",\n  data: %{\"apiVersion\" => \"v1\", \"kind\" => \"Scale\"}, # `data` is of kind \"Scale\"\n  path_params: [name: \"nginx\", namespace: \"default\"],\n  header_params: [\"Content-Type\": \"application/json\"]\n}\n```\n\nThe following example would `update` the *nginx* deployment's `Status`. Note the `deployments/status` operation will have a `Deployment` *data* payload:\n\n```elixir\n%K8s.Operation{\n  method: :put,\n  verb: :update,\n  api_version: \"apps/v1\", # api version of the \"Deployment\" kind\n  name: \"deployments/status\",\n  data: %{\"apiVersion\" => \"apps/v1\", \"kind\" => \"Deployment\"}, # `data` is of kind \"Deployment\"\n  path_params: [name: \"nginx\", namespace: \"default\"],\n  header_params: [\"Content-Type\": \"application/json\"]\n}\n```","ref":"K8s.Operation.html#t:t/0"},{"type":"module","title":"K8s.Operation.Path","doc":"Generates Kubernetes REST API Paths","ref":"K8s.Operation.Path.html"},{"type":"function","title":"K8s.Operation.Path.build/1","doc":"Generates the API path for a `K8s.Operation`.","ref":"K8s.Operation.Path.html#build/1"},{"type":"function","title":"Examples - K8s.Operation.Path.build/1","doc":"Generate a path for a cluster scoped resource:\n\n    iex> resource = K8s.Resource.build(\"apps/v1\", \"CertificateSigningRequest\", \"foo\")\n    ...> operation = %K8s.Operation{\n    ...>  method: :put,\n    ...>  verb: :update,\n    ...>  data: resource,\n    ...>  path_params: [name: \"foo\"],\n    ...>  api_version: \"apps/v1\",\n    ...>  name: \"certificatesigningrequests\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/apis/apps/v1/certificatesigningrequests/foo\"}\n\n\n    iex> resource = K8s.Resource.build(\"apps/v1\", \"CertificateSigningRequest\", \"foo\")\n    ...> operation = %K8s.Operation{\n    ...>  method: :put,\n    ...>  verb: :update,\n    ...>  data: resource,\n    ...>  path_params: [name: \"foo\", namespace: nil],\n    ...>  api_version: \"apps/v1\",\n    ...>  name: \"certificatesigningrequests\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/apis/apps/v1/certificatesigningrequests/foo\"}\n\nGenerate a path for a namespace scoped resource:\n\n    iex> resource = K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"foo\")\n    ...> operation = %K8s.Operation{\n    ...>  method: :put,\n    ...>  verb: :update,\n    ...>  data: resource,\n    ...>  path_params: [namespace: \"default\", name: \"foo\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/namespaces/default/pods/foo\"}\n\nGenerate a path for a namespace scoped resource on the collection: (ie create, list)\n\n    iex> resource = K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"foo\")\n    ...> operation = %K8s.Operation{\n    ...>  method: :post,\n    ...>  verb: :create,\n    ...>  data: resource,\n    ...>  path_params: [namespace: \"default\", name: \"foo\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/namespaces/default/pods\"}\n\nGenerating a listing path for a namespace:\n\n    iex> operation = %K8s.Operation{\n    ...>  method: :get,\n    ...>  verb: :list,\n    ...>  data: nil,\n    ...>  path_params: [namespace: \"default\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/namespaces/default/pods\"}\n\nGenerating a listing path for a all namespaces:\n\n    iex> operation = %K8s.Operation{\n    ...>  method: :get,\n    ...>  verb: :list_all_namespaces,\n    ...>  data: nil,\n    ...>  path_params: [namespace: \"default\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/pods\"}\n\nGenerating a listing path for a watching all namespaces:\n\n    iex> operation = %K8s.Operation{\n    ...>  method: :get,\n    ...>  verb: :watch_all_namespaces,\n    ...>  data: nil,\n    ...>  path_params: [namespace: \"default\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/pods\"}\n\nGenerating a path for a subresource:\n\n    iex> operation = %K8s.Operation{\n    ...>  method: :get,\n    ...>  verb: :get,\n    ...>  data: nil,\n    ...>  path_params: [namespace: \"default\", name: \"foo\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods/status\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/namespaces/default/pods/foo/status\"}\n\nDeleting a collection:\n\n    iex> operation = %K8s.Operation{\n    ...>  method: :delete,\n    ...>  verb: :deletecollection,\n    ...>  data: nil,\n    ...>  path_params: [namespace: \"default\"],\n    ...>  api_version: \"v1\",\n    ...>  name: \"pods\"\n    ...> }\n    ...> K8s.Operation.Path.build(operation)\n    {:ok, \"/api/v1/namespaces/default/pods\"}","ref":"K8s.Operation.Path.html#build/1-examples"},{"type":"function","title":"K8s.Operation.Path.find_params/1","doc":"Find valid path params in a URL path.","ref":"K8s.Operation.Path.html#find_params/1"},{"type":"function","title":"Examples - K8s.Operation.Path.find_params/1","doc":"iex> K8s.Operation.Path.find_params(\"/foo/{name}\")\n    [:name]\n\n    iex> K8s.Operation.Path.find_params(\"/foo/{namespace}/bar/{name}\")\n    [:namespace, :name]\n\n    iex> K8s.Operation.Path.find_params(\"/foo/bar\")\n    []","ref":"K8s.Operation.Path.html#find_params/1-examples"},{"type":"function","title":"K8s.Operation.Path.replace_path_vars/2","doc":"Replaces path variables with options.","ref":"K8s.Operation.Path.html#replace_path_vars/2"},{"type":"function","title":"Examples - K8s.Operation.Path.replace_path_vars/2","doc":"iex> K8s.Operation.Path.replace_path_vars(\"/foo/{name}\", name: \"bar\")\n    \"/foo/bar\"","ref":"K8s.Operation.Path.html#replace_path_vars/2-examples"},{"type":"module","title":"K8s.Resource","doc":"Kubernetes manifest attribute helpers","ref":"K8s.Resource.html"},{"type":"function","title":"K8s.Resource.all_from_file/2","doc":"Create a list of resource `Map`s from a YAML file containing a YAML stream.","ref":"K8s.Resource.html#all_from_file/2"},{"type":"function","title":"Examples - K8s.Resource.all_from_file/2","doc":"iex> opts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\n    ...> K8s.Resource.all_from_file(\"test/support/helm-chart.yaml\", opts)\n    {:ok, [\n      %{\n        \"apiVersion\" => \"v1\",\n        \"kind\" => \"Namespace\",\n        \"metadata\" => %{\"name\" => \"default\"}\n      },\n      %{\n        \"apiVersion\" => \"apps/v1\",\n        \"kind\" => \"Deployment\",\n        \"metadata\" => %{\n          \"labels\" => %{\"app\" => \"nginx\"},\n          \"name\" => \"nginx-deployment\",\n          \"namespace\" => \"default\"\n        },\n        \"spec\" => %{\n          \"replicas\" => 3,\n          \"selector\" => %{\"matchLabels\" => %{\"app\" => \"nginx\"}},\n          \"template\" => %{\n            \"metadata\" => %{\"labels\" => %{\"app\" => \"nginx\"}},\n            \"spec\" => %{\n              \"containers\" => [\n                %{\n                  \"image\" => \"nginx:nginx:1.7.9\",\n                  \"name\" => \"nginx\",\n                  \"ports\" => [%{\"containerPort\" => 80}]\n                }\n              ]\n            }\n          }\n        }\n      }\n    ]}","ref":"K8s.Resource.html#all_from_file/2-examples"},{"type":"function","title":"K8s.Resource.all_from_file!/2","doc":"Create a list of resource `Map`s from a YAML file containing a YAML stream.\n\nRaises `File.Error` when the file does not exist.","ref":"K8s.Resource.html#all_from_file!/2"},{"type":"function","title":"Examples - K8s.Resource.all_from_file!/2","doc":"iex> opts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\n    ...> K8s.Resource.all_from_file!(\"test/support/helm-chart.yaml\", opts)\n    [\n      %{\n        \"apiVersion\" => \"v1\",\n        \"kind\" => \"Namespace\",\n        \"metadata\" => %{\"name\" => \"default\"}\n      },\n      %{\n        \"apiVersion\" => \"apps/v1\",\n        \"kind\" => \"Deployment\",\n        \"metadata\" => %{\n          \"labels\" => %{\"app\" => \"nginx\"},\n          \"name\" => \"nginx-deployment\",\n          \"namespace\" => \"default\"\n        },\n        \"spec\" => %{\n          \"replicas\" => 3,\n          \"selector\" => %{\"matchLabels\" => %{\"app\" => \"nginx\"}},\n          \"template\" => %{\n            \"metadata\" => %{\"labels\" => %{\"app\" => \"nginx\"}},\n            \"spec\" => %{\n              \"containers\" => [\n                %{\n                  \"image\" => \"nginx:nginx:1.7.9\",\n                  \"name\" => \"nginx\",\n                  \"ports\" => [%{\"containerPort\" => 80}]\n                }\n              ]\n            }\n          }\n        }\n      }\n    ]","ref":"K8s.Resource.html#all_from_file!/2-examples"},{"type":"function","title":"K8s.Resource.annotation/2","doc":"","ref":"K8s.Resource.html#annotation/2"},{"type":"function","title":"K8s.Resource.annotations/1","doc":"","ref":"K8s.Resource.html#annotations/1"},{"type":"function","title":"K8s.Resource.api_version/1","doc":"","ref":"K8s.Resource.html#api_version/1"},{"type":"function","title":"K8s.Resource.build/2","doc":"Helper for building a kubernetes' resource `Map`","ref":"K8s.Resource.html#build/2"},{"type":"function","title":"Examples - K8s.Resource.build/2","doc":"iex> K8s.Resource.build(\"v1\", \"Pod\")\n    %{\"apiVersion\" => \"v1\", \"kind\" => \"Pod\", \"metadata\" => %{}}\n\n    iex> K8s.Resource.build(\"v1\", \"Namespace\", \"foo\")\n    %{\"apiVersion\" => \"v1\", \"kind\" => \"Namespace\", \"metadata\" => %{\"name\" => \"foo\"}}\n\n    iex> K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"foo\")\n    %{\"apiVersion\" => \"v1\", \"kind\" => \"Pod\", \"metadata\" => %{\"namespace\" => \"default\", \"name\" => \"foo\"}}","ref":"K8s.Resource.html#build/2-examples"},{"type":"function","title":"K8s.Resource.build/3","doc":"","ref":"K8s.Resource.html#build/3"},{"type":"function","title":"K8s.Resource.build/4","doc":"","ref":"K8s.Resource.html#build/4"},{"type":"function","title":"K8s.Resource.build/5","doc":"","ref":"K8s.Resource.html#build/5"},{"type":"function","title":"K8s.Resource.cpu/1","doc":"","ref":"K8s.Resource.html#cpu/1"},{"type":"function","title":"K8s.Resource.from_file/2","doc":"Create a resource `Map` from a YAML file.","ref":"K8s.Resource.html#from_file/2"},{"type":"function","title":"Examples - K8s.Resource.from_file/2","doc":"iex> opts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\n    ...> K8s.Resource.from_file(\"test/support/deployment.yaml\", opts)\n    {:ok, %{\n      \"apiVersion\" => \"apps/v1\",\n      \"kind\" => \"Deployment\",\n      \"metadata\" => %{\n        \"labels\" => %{\"app\" => \"nginx\"},\n        \"name\" => \"nginx-deployment\",\n        \"namespace\" => \"default\"\n      },\n      \"spec\" => %{\n        \"replicas\" => 3,\n        \"selector\" => %{\"matchLabels\" => %{\"app\" => \"nginx\"}},\n        \"template\" => %{\n          \"metadata\" => %{\"labels\" => %{\"app\" => \"nginx\"}},\n          \"spec\" => %{\n            \"containers\" => [\n              %{\n                \"image\" => \"nginx:nginx:1.7.9\",\n                \"name\" => \"nginx\",\n                \"ports\" => [%{\"containerPort\" => 80}]\n              }\n            ]\n          }\n        }\n      }\n    }}","ref":"K8s.Resource.html#from_file/2-examples"},{"type":"function","title":"K8s.Resource.from_file!/2","doc":"Create a resource `Map` from a YAML file.\n\nRaises `File.Error` when the file does not exist.","ref":"K8s.Resource.html#from_file!/2"},{"type":"function","title":"Examples - K8s.Resource.from_file!/2","doc":"iex> opts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\n    ...> K8s.Resource.from_file!(\"test/support/deployment.yaml\", opts)\n    %{\n      \"apiVersion\" => \"apps/v1\",\n      \"kind\" => \"Deployment\",\n      \"metadata\" => %{\n        \"labels\" => %{\"app\" => \"nginx\"},\n        \"name\" => \"nginx-deployment\",\n        \"namespace\" => \"default\"\n      },\n      \"spec\" => %{\n        \"replicas\" => 3,\n        \"selector\" => %{\"matchLabels\" => %{\"app\" => \"nginx\"}},\n        \"template\" => %{\n          \"metadata\" => %{\"labels\" => %{\"app\" => \"nginx\"}},\n          \"spec\" => %{\n            \"containers\" => [\n              %{\n                \"image\" => \"nginx:nginx:1.7.9\",\n                \"name\" => \"nginx\",\n                \"ports\" => [%{\"containerPort\" => 80}]\n              }\n            ]\n          }\n        }\n      }\n    }","ref":"K8s.Resource.html#from_file!/2-examples"},{"type":"function","title":"K8s.Resource.has_annotation?/2","doc":"","ref":"K8s.Resource.html#has_annotation?/2"},{"type":"function","title":"K8s.Resource.has_label?/2","doc":"","ref":"K8s.Resource.html#has_label?/2"},{"type":"function","title":"K8s.Resource.kind/1","doc":"","ref":"K8s.Resource.html#kind/1"},{"type":"function","title":"K8s.Resource.label/2","doc":"","ref":"K8s.Resource.html#label/2"},{"type":"function","title":"K8s.Resource.labels/1","doc":"","ref":"K8s.Resource.html#labels/1"},{"type":"function","title":"K8s.Resource.memory/1","doc":"","ref":"K8s.Resource.html#memory/1"},{"type":"function","title":"K8s.Resource.metadata/1","doc":"","ref":"K8s.Resource.html#metadata/1"},{"type":"function","title":"K8s.Resource.name/1","doc":"","ref":"K8s.Resource.html#name/1"},{"type":"function","title":"K8s.Resource.namespace/1","doc":"","ref":"K8s.Resource.html#namespace/1"},{"type":"type","title":"K8s.Resource.yaml_elixir_error_t/0","doc":"","ref":"K8s.Resource.html#t:yaml_elixir_error_t/0"},{"type":"module","title":"K8s.Resource.FieldAccessors","doc":"Helper functions for accessing common fields","ref":"K8s.Resource.FieldAccessors.html"},{"type":"function","title":"K8s.Resource.FieldAccessors.annotation/2","doc":"Returns the value of a k8s resource's annotation.","ref":"K8s.Resource.FieldAccessors.html#annotation/2"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.annotation/2","doc":"iex> K8s.Resource.FieldAccessors.annotation(%{\"metadata\" => %{\"annotations\" => %{\"env\" => \"test\"}}}, \"env\")\n    \"test\"","ref":"K8s.Resource.FieldAccessors.html#annotation/2-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.annotations/1","doc":"Returns the annotations of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#annotations/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.annotations/1","doc":"iex> K8s.Resource.FieldAccessors.annotations(%{\"metadata\" => %{\"annotations\" => %{\"env\" => \"test\"}}})\n    %{\"env\" => \"test\"}","ref":"K8s.Resource.FieldAccessors.html#annotations/1-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.api_version/1","doc":"Returns the apiVersion of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#api_version/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.api_version/1","doc":"iex> K8s.Resource.FieldAccessors.api_version(%{\"apiVersion\" => \"apps/v1\"})\n    \"apps/v1\"","ref":"K8s.Resource.FieldAccessors.html#api_version/1-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.has_annotation?/2","doc":"Check if an annotation is present.","ref":"K8s.Resource.FieldAccessors.html#has_annotation?/2"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.has_annotation?/2","doc":"iex> K8s.Resource.FieldAccessors.has_annotation?(%{\"metadata\" => %{\"annotations\" => %{\"env\" => \"test\"}}}, \"env\")\n    true\n\n    iex> K8s.Resource.FieldAccessors.has_annotation?(%{\"metadata\" => %{\"annotations\" => %{\"env\" => \"test\"}}}, \"foo\")\n    false","ref":"K8s.Resource.FieldAccessors.html#has_annotation?/2-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.has_label?/2","doc":"Check if a label is present.","ref":"K8s.Resource.FieldAccessors.html#has_label?/2"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.has_label?/2","doc":"iex> K8s.Resource.FieldAccessors.has_label?(%{\"metadata\" => %{\"labels\" => %{\"env\" => \"test\"}}}, \"env\")\n    true\n\n    iex> K8s.Resource.FieldAccessors.has_label?(%{\"metadata\" => %{\"labels\" => %{\"env\" => \"test\"}}}, \"foo\")\n    false","ref":"K8s.Resource.FieldAccessors.html#has_label?/2-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.kind/1","doc":"Returns the kind of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#kind/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.kind/1","doc":"iex> K8s.Resource.FieldAccessors.kind(%{\"kind\" => \"Deployment\"})\n    \"Deployment\"","ref":"K8s.Resource.FieldAccessors.html#kind/1-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.label/2","doc":"Returns the value of a k8s resource's label.","ref":"K8s.Resource.FieldAccessors.html#label/2"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.label/2","doc":"iex> K8s.Resource.FieldAccessors.label(%{\"metadata\" => %{\"labels\" => %{\"env\" => \"test\"}}}, \"env\")\n    \"test\"","ref":"K8s.Resource.FieldAccessors.html#label/2-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.labels/1","doc":"Returns the labels of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#labels/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.labels/1","doc":"iex> K8s.Resource.FieldAccessors.labels(%{\"metadata\" => %{\"labels\" => %{\"env\" => \"test\"}}})\n    %{\"env\" => \"test\"}","ref":"K8s.Resource.FieldAccessors.html#labels/1-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.metadata/1","doc":"Returns the metadata of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#metadata/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.metadata/1","doc":"iex> K8s.Resource.FieldAccessors.metadata(%{\"metadata\" => %{\"name\" => \"nginx\", \"namespace\" => \"foo\"}})\n    %{\"name\" => \"nginx\", \"namespace\" => \"foo\"}","ref":"K8s.Resource.FieldAccessors.html#metadata/1-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.name/1","doc":"Returns the name of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#name/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.name/1","doc":"iex> K8s.Resource.FieldAccessors.name(%{\"metadata\" => %{\"name\" => \"nginx\", \"namespace\" => \"foo\"}})\n    \"nginx\"","ref":"K8s.Resource.FieldAccessors.html#name/1-examples"},{"type":"function","title":"K8s.Resource.FieldAccessors.namespace/1","doc":"Returns the namespace of k8s resource.","ref":"K8s.Resource.FieldAccessors.html#namespace/1"},{"type":"function","title":"Examples - K8s.Resource.FieldAccessors.namespace/1","doc":"iex> K8s.Resource.FieldAccessors.namespace(%{\"metadata\" => %{\"name\" => \"nginx\", \"namespace\" => \"foo\"}})\n    \"foo\"","ref":"K8s.Resource.FieldAccessors.html#namespace/1-examples"},{"type":"module","title":"K8s.Resource.NamedList","doc":"Provides an accessor to a list of maps whereas each element in the list has a key named \"name\". The name should\nbe unique within the list and therefore defining the element.","ref":"K8s.Resource.NamedList.html"},{"type":"function","title":"K8s.Resource.NamedList.access/1","doc":"Provides an accessor to a list of maps whereas each element in the list has a key named \"name\". If no such element\nexists within the given list, get_in() will return nil while updating will create an empty element with the given\nname.","ref":"K8s.Resource.NamedList.html#access/1"},{"type":"function","title":"Examples - K8s.Resource.NamedList.access/1","doc":"iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access(\"key2\"), \"value\"])\n  \"value2\"\n\n  iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access(\"key2\")])\n  %{\"name\" => \"key2\", \"value\" => \"value2\"}\n\n  iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access(\"missing-key\"), \"value\"])\n  nil\n\n  iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access(\"missing-key\")])\n  nil\n\n  iex> put_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access(\"key1\"), \"value\"], \"value_new\")\n  [%{\"name\" => \"key1\", \"value\" => \"value_new\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}]\n\n  iex> put_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access(\"missing-key\"), \"value\"], \"value_new\")\n  [%{\"name\" => \"missing-key\", \"value\" => \"value_new\"}, %{\"name\" => \"key1\", \"value\" => \"value1\"}]\n\n  iex> pop_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access(\"key1\"), \"value\"])\n  {\"value1\", [%{\"name\" => \"key1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}]}\n\n  iex> pop_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access(\"missing-key\"), \"value\"])\n  {nil, [%{\"name\" => \"missing-key\"}, %{\"name\" => \"key1\", \"value\" => \"value1\"}]}\n\n  iex> pop_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access(\"missing-key\")])\n  {%{\"name\" => \"missing-key\"}, [%{\"name\" => \"key1\", \"value\" => \"value1\"}]}","ref":"K8s.Resource.NamedList.html#access/1-examples"},{"type":"function","title":"K8s.Resource.NamedList.access!/1","doc":"Provides an accessor to a list of maps whereas each element in the list has a key named \"name\". If no such element\nexists within the given list, an exception is raised.","ref":"K8s.Resource.NamedList.html#access!/1"},{"type":"function","title":"Examples - K8s.Resource.NamedList.access!/1","doc":"iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access!(\"key2\"), \"value\"])\n  \"value2\"\n\n  iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access!(\"key2\")])\n  %{\"name\" => \"key2\", \"value\" => \"value2\"}\n\n  iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access!(\"missing-key\"), \"value\"])\n  ** (ArgumentError) There is not item with name missing-key in the given named list.\n\n  iex> get_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access!(\"missing-key\")])\n  ** (ArgumentError) There is not item with name missing-key in the given named list.\n\n  iex> put_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access!(\"key1\"), \"value\"], \"value_new\")\n  [%{\"name\" => \"key1\", \"value\" => \"value_new\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}]\n\n  iex> put_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access!(\"missing-key\"), \"value\"], \"value_new\")\n  ** (ArgumentError) There is not item with name missing-key in the given named list.\n\n  iex> pop_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}], [K8s.Resource.NamedList.access!(\"key1\"), \"value\"])\n  {\"value1\", [%{\"name\" => \"key1\"}, %{\"name\" => \"key2\", \"value\" => \"value2\"}]}\n\n  iex> pop_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access!(\"missing-key\"), \"value\"])\n  ** (ArgumentError) There is not item with name missing-key in the given named list.\n\n  iex> pop_in([%{\"name\" => \"key1\", \"value\" => \"value1\"}], [K8s.Resource.NamedList.access!(\"missing-key\")])\n  ** (ArgumentError) There is not item with name missing-key in the given named list.","ref":"K8s.Resource.NamedList.html#access!/1-examples"},{"type":"module","title":"K8s.Resource.Utilization","doc":"Deserializers for CPU and Memory values","ref":"K8s.Resource.Utilization.html"},{"type":"function","title":"K8s.Resource.Utilization.cpu/1","doc":"Deserializes CPU quantity","ref":"K8s.Resource.Utilization.html#cpu/1"},{"type":"function","title":"Examples - K8s.Resource.Utilization.cpu/1","doc":"Parses whole values\n    iex> K8s.Resource.Utilization.cpu(\"3\")\n    3\n\n  Parses millicpu values\n    iex> K8s.Resource.Utilization.cpu(\"500m\")\n    0.5\n\n  Parses decimal values\n    iex> K8s.Resource.Utilization.cpu(\"1.5\")\n    1.5","ref":"K8s.Resource.Utilization.html#cpu/1-examples"},{"type":"function","title":"K8s.Resource.Utilization.memory/1","doc":"Deserializes memory quantity","ref":"K8s.Resource.Utilization.html#memory/1"},{"type":"function","title":"Examples - K8s.Resource.Utilization.memory/1","doc":"Parses whole values\n    iex> K8s.Resource.Utilization.memory(\"1000000\")\n    1000000\n\n  Parses decimal values\n    iex> K8s.Resource.Utilization.memory(\"10.75\")\n    10.75\n\n  Parses decimalSI values\n    iex> K8s.Resource.Utilization.memory(\"10M\")\n    10000000\n\n  Parses binarySI suffixes\n    iex> K8s.Resource.Utilization.memory(\"50Mi\")\n    52428800\n\n  Returns the numeric value when the suffix is unrecognized\n    iex> K8s.Resource.Utilization.memory(\"50Foo\")\n    50","ref":"K8s.Resource.Utilization.html#memory/1-examples"},{"type":"module","title":"K8s.Selector","doc":"Builds [label selectors](https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/) and [field selectors](https://kubernetes.io/docs/concepts/overview/working-with-objects/field-selectors/) for `K8s.Operation`s","ref":"K8s.Selector.html"},{"type":"module","title":"Examples - K8s.Selector","doc":"Parse from a YAML map\n\n    iex> deployment = %{\n    ...>   \"kind\" => \"Deployment\",\n    ...>   \"metadata\" => %{\n    ...>     \"name\" => \"nginx\",\n    ...>     \"labels\" => %{\n    ...>       \"app\" => \"nginx\",\n    ...>       \"tier\" => \"backend\"\n    ...>     }\n    ...>   },\n    ...>   \"spec\" => %{\n    ...>     \"selector\" => %{\n    ...>       \"matchLabels\" => %{\n    ...>         \"app\" => \"nginx\"\n    ...>       }\n    ...>     },\n    ...>     \"template\" => %{\n    ...>       \"metadata\" => %{\n    ...>         \"labels\" => %{\n    ...>           \"app\" => \"nginx\",\n    ...>           \"tier\" => \"backend\"\n    ...>         }\n    ...>       }\n    ...>     }\n    ...>   }\n    ...> }\n    ...> K8s.Selector.parse(deployment)\n    %K8s.Selector{match_labels: %{\"app\" => \"nginx\"}}\n\n  Provides a composable interface for building label selectors\n\n    iex> {\"component\", \"redis\"}\n    ...> |> K8s.Selector.label()\n    ...> |> K8s.Selector.label_not({\"foo\", \"bar\"})\n    ...> |> K8s.Selector.label_in({\"tier\", \"cache\"})\n    ...> |> K8s.Selector.label_not_in({\"environment\", \"dev\"})\n    ...> |> K8s.Selector.label_exists(\"foo\")\n    ...> |> K8s.Selector.label_does_not_exist(\"bar\")\n    %K8s.Selector{\n      match_labels: %{\"component\" => {\"=\", \"redis\"}, \"foo\" => {\"!=\", \"bar\"}},\n      match_expressions: [\n        %{\"key\" => \"bar\", \"operator\" => \"DoesNotExist\"},\n        %{\"key\" => \"foo\", \"operator\" => \"Exists\"},\n        %{\"key\" => \"environment\", \"operator\" => \"NotIn\", \"values\" => [\"dev\"]},\n        %{\"key\" => \"tier\", \"operator\" => \"In\", \"values\" => [\"cache\"]}\n      ]\n    }\n\n  Provides a composable interface for adding selectors to `K8s.Operation`s.\n\n    iex> K8s.Client.get(\"v1\", :pods)\n    ...> |> K8s.Selector.label({\"app\", \"nginx\"})\n    ...> |> K8s.Selector.label_not(%{\"tier\" => \"backend\"})\n    ...> |> K8s.Selector.label_in({\"environment\", [\"qa\", \"prod\"]})\n    %K8s.Operation{data: nil, api_version: \"v1\", query_params: [labelSelector: %K8s.Selector{match_expressions: [%{\"key\" => \"environment\", \"operator\" => \"In\", \"values\" => [\"qa\", \"prod\"]}], match_labels: %{\"app\" => {\"=\", \"nginx\"}, \"tier\" => {\"!=\", \"backend\"}}}], method: :get, name: :pods, path_params: [], verb: :get}","ref":"K8s.Selector.html#module-examples"},{"type":"function","title":"K8s.Selector.field/1","doc":"`fieldSelector` helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#field/1"},{"type":"function","title":"Examples - K8s.Selector.field/1","doc":"iex> K8s.Selector.field({\"metadata.namespace\", \"default\"})\n    %K8s.Selector{match_fields: %{\"metadata.namespace\" => {\"=\", \"default\"}}}\n\n    iex> K8s.Selector.field(%{\"metadata.namespace\" => \"default\", \"status.phase\" => \"Running\"})\n    %K8s.Selector{match_fields: %{\"metadata.namespace\" => {\"=\", \"default\"}, \"status.phase\" => {\"=\", \"Running\"}}}","ref":"K8s.Selector.html#field/1-examples"},{"type":"function","title":"K8s.Selector.field/2","doc":"","ref":"K8s.Selector.html#field/2"},{"type":"function","title":"K8s.Selector.field_not/1","doc":"`fieldSelector` helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#field_not/1"},{"type":"function","title":"Examples - K8s.Selector.field_not/1","doc":"iex> K8s.Selector.field_not({\"metadata.namespace\", \"default\"})\n    %K8s.Selector{match_fields: %{\"metadata.namespace\" => {\"!=\", \"default\"}}}\n\n    iex> K8s.Selector.field(%{\"metadata.namespace\" => \"default\", \"status.phase\" => \"Running\"})\n    %K8s.Selector{match_fields: %{\"metadata.namespace\" => {\"=\", \"default\"}, \"status.phase\" => {\"=\", \"Running\"}}}","ref":"K8s.Selector.html#field_not/1-examples"},{"type":"function","title":"K8s.Selector.field_not/2","doc":"","ref":"K8s.Selector.html#field_not/2"},{"type":"function","title":"K8s.Selector.fields_to_s/1","doc":"Serializes a `K8s.Selector` to a `fieldSelector` query string.","ref":"K8s.Selector.html#fields_to_s/1"},{"type":"function","title":"Examples - K8s.Selector.fields_to_s/1","doc":"iex> selector = K8s.Selector.field({\"status.phase\", \"Running\"})\n    ...> K8s.Selector.fields_to_s(selector)\n    \"status.phase=Running\"","ref":"K8s.Selector.html#fields_to_s/1-examples"},{"type":"function","title":"K8s.Selector.label/1","doc":"`matchLabels` helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label/1"},{"type":"function","title":"Examples - K8s.Selector.label/1","doc":"iex> K8s.Selector.label({\"component\", \"redis\"})\n    %K8s.Selector{match_labels: %{\"component\" => {\"=\", \"redis\"}}}\n\n    iex> K8s.Selector.label(%{\"component\" => \"redis\", \"env\" => \"prod\"})\n    %K8s.Selector{match_labels: %{\"component\" => {\"=\", \"redis\"}, \"env\" => {\"=\", \"prod\"}}}","ref":"K8s.Selector.html#label/1-examples"},{"type":"function","title":"K8s.Selector.label/2","doc":"`matchLabels` helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label/2"},{"type":"function","title":"Examples - K8s.Selector.label/2","doc":"iex> K8s.Selector.label({\"component\", \"redis\"})\n    ...> |> K8s.Selector.label({\"environment\", \"dev\"})\n    %K8s.Selector{match_labels: %{\"component\" => {\"=\", \"redis\"}, \"environment\" => {\"=\", \"dev\"}}}","ref":"K8s.Selector.html#label/2-examples"},{"type":"function","title":"K8s.Selector.label_does_not_exist/1","doc":"`DoesNotExist` expression helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label_does_not_exist/1"},{"type":"function","title":"Examples - K8s.Selector.label_does_not_exist/1","doc":"iex> K8s.Selector.label_does_not_exist(\"environment\")\n    %K8s.Selector{match_expressions: [%{\"key\" => \"environment\", \"operator\" => \"DoesNotExist\"}]}","ref":"K8s.Selector.html#label_does_not_exist/1-examples"},{"type":"function","title":"K8s.Selector.label_does_not_exist/2","doc":"","ref":"K8s.Selector.html#label_does_not_exist/2"},{"type":"function","title":"K8s.Selector.label_exists/1","doc":"`Exists` expression helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label_exists/1"},{"type":"function","title":"Examples - K8s.Selector.label_exists/1","doc":"iex> K8s.Selector.label_exists(\"environment\")\n    %K8s.Selector{match_expressions: [%{\"key\" => \"environment\", \"operator\" => \"Exists\"}]}","ref":"K8s.Selector.html#label_exists/1-examples"},{"type":"function","title":"K8s.Selector.label_exists/2","doc":"","ref":"K8s.Selector.html#label_exists/2"},{"type":"function","title":"K8s.Selector.label_in/1","doc":"`In` expression helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label_in/1"},{"type":"function","title":"Examples - K8s.Selector.label_in/1","doc":"iex> K8s.Selector.label_in({\"component\", \"redis\"})\n    %K8s.Selector{match_expressions: [%{\"key\" => \"component\", \"operator\" => \"In\", \"values\" => [\"redis\"]}]}","ref":"K8s.Selector.html#label_in/1-examples"},{"type":"function","title":"K8s.Selector.label_in/2","doc":"","ref":"K8s.Selector.html#label_in/2"},{"type":"function","title":"K8s.Selector.label_not/1","doc":"`matchLabels` helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label_not/1"},{"type":"function","title":"Examples - K8s.Selector.label_not/1","doc":"iex> K8s.Selector.label_not({\"component\", \"redis\"})\n    %K8s.Selector{match_labels: %{\"component\" => {\"!=\", \"redis\"}}}\n\n    iex> K8s.Selector.label_not(%{\"component\" => \"redis\", \"env\" => \"prod\"})\n    %K8s.Selector{match_labels: %{\"component\" => {\"!=\", \"redis\"}, \"env\" => {\"!=\", \"prod\"}}}","ref":"K8s.Selector.html#label_not/1-examples"},{"type":"function","title":"K8s.Selector.label_not/2","doc":"`matchLabels` helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label_not/2"},{"type":"function","title":"Examples - K8s.Selector.label_not/2","doc":"iex> K8s.Selector.label_not({\"component\", \"redis\"})\n    ...> |> K8s.Selector.label_not({\"environment\", \"dev\"})\n    %K8s.Selector{match_labels: %{\"component\" => {\"!=\", \"redis\"}, \"environment\" => {\"!=\", \"dev\"}}}","ref":"K8s.Selector.html#label_not/2-examples"},{"type":"function","title":"K8s.Selector.label_not_in/1","doc":"`NotIn` expression helper that creates a composable `K8s.Selector`.","ref":"K8s.Selector.html#label_not_in/1"},{"type":"function","title":"Examples - K8s.Selector.label_not_in/1","doc":"iex> K8s.Selector.label_not_in({\"component\", \"redis\"})\n    %K8s.Selector{match_expressions: [%{\"key\" => \"component\", \"operator\" => \"NotIn\", \"values\" => [\"redis\"]}]}","ref":"K8s.Selector.html#label_not_in/1-examples"},{"type":"function","title":"K8s.Selector.label_not_in/2","doc":"","ref":"K8s.Selector.html#label_not_in/2"},{"type":"function","title":"K8s.Selector.labels_to_s/1","doc":"Serializes a `K8s.Selector` to a `labelSelector` query string.","ref":"K8s.Selector.html#labels_to_s/1"},{"type":"function","title":"Examples - K8s.Selector.labels_to_s/1","doc":"iex> selector = K8s.Selector.label({\"component\", \"redis\"})\n    ...> K8s.Selector.labels_to_s(selector)\n    \"component=redis\"","ref":"K8s.Selector.html#labels_to_s/1-examples"},{"type":"function","title":"K8s.Selector.match_expression?/2","doc":"Checks whether a resource matches a single selector `matchExpressions`","ref":"K8s.Selector.html#match_expression?/2"},{"type":"function","title":"Examples - K8s.Selector.match_expression?/2","doc":"When an `In` expression matches\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"prod\"}}}\n    ...> expr = %{\"operator\" => \"In\", \"key\" => \"env\", \"values\" => [\"prod\", \"qa\"]}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    true\n\n  When an `In` expression doesnt match\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"In\", \"key\" => \"env\", \"values\" => [\"prod\", \"qa\"]}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    false\n\n  When an `NotIn` expression matches\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"NotIn\", \"key\" => \"env\", \"values\" => [\"prod\"]}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    true\n\n  When an `NotIn` expression doesnt match\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"NotIn\", \"key\" => \"env\", \"values\" => [\"dev\"]}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    false\n\n  When an `Exists` expression matches\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"Exists\", \"key\" => \"env\"}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    true\n\n  When an `Exists` expression doesnt match\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"Exists\", \"key\" => \"tier\"}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    false\n\n  When an `DoesNotExist` expression matches\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"DoesNotExist\", \"key\" => \"tier\"}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    true\n\n  When an `DoesNotExist` expression doesnt match\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"dev\"}}}\n    ...> expr = %{\"operator\" => \"DoesNotExist\", \"key\" => \"env\"}\n    ...> K8s.Selector.match_expression?(resource, expr)\n    false","ref":"K8s.Selector.html#match_expression?/2-examples"},{"type":"function","title":"K8s.Selector.match_expressions?/2","doc":"Checks if a `K8s.Resource` matches all `matchExpressions` using a logical `AND`","ref":"K8s.Selector.html#match_expressions?/2"},{"type":"function","title":"Examples - K8s.Selector.match_expressions?/2","doc":"Accepts `K8s.Selector`s:\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"prod\", \"tier\" => \"frontend\"}}}\n    ...> expr1 = %{\"operator\" => \"In\", \"key\" => \"env\", \"values\" => [\"prod\", \"qa\"]}\n    ...> expr2 = %{\"operator\" => \"Exists\", \"key\" => \"tier\"}\n    ...> selector = %K8s.Selector{match_expressions: [expr1, expr2]}\n    ...> K8s.Selector.match_expressions?(resource, selector)\n    true\n\n  Accepts `map`s:\n\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"prod\", \"tier\" => \"frontend\"}}}\n    ...> expr1 = %{\"operator\" => \"In\", \"key\" => \"env\", \"values\" => [\"prod\", \"qa\"]}\n    ...> expr2 = %{\"operator\" => \"Exists\", \"key\" => \"tier\"}\n    ...> K8s.Selector.match_expressions?(resource, [expr1, expr2])\n    true\n\n  Returns `false` when not matching all expressions:\n\n    iex> resource = %{\"kind\" => \"Node\", \"metadata\" => %{\"labels\" => %{\"env\" => \"prod\", \"tier\" => \"frontend\"}}}\n    ...> expr1 = %{\"operator\" => \"In\", \"key\" => \"env\", \"values\" => [\"prod\", \"qa\"]}\n    ...> expr2 = %{\"operator\" => \"Exists\", \"key\" => \"foo\"}\n    ...> K8s.Selector.match_expressions?(resource, [expr1, expr2])\n    false","ref":"K8s.Selector.html#match_expressions?/2-examples"},{"type":"function","title":"K8s.Selector.match_label?/3","doc":"Checks if a `K8s.Resource` matches a single label","ref":"K8s.Selector.html#match_label?/3"},{"type":"function","title":"K8s.Selector.match_labels?/2","doc":"Checks if a `K8s.Resource` matches all `matchLabels` using a logcal `AND`","ref":"K8s.Selector.html#match_labels?/2"},{"type":"function","title":"Examples - K8s.Selector.match_labels?/2","doc":"Accepts `K8s.Selector`s:\n    iex> labels = %{\"env\" => \"prod\", \"tier\" => \"frontend\"}\n    ...> selector = %K8s.Selector{match_labels: labels}\n    ...> resource = K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"test\", labels)\n    ...> K8s.Selector.match_labels?(resource, selector)\n    true\n\n  Accepts maps:\n    iex> labels = %{\"env\" => \"prod\", \"tier\" => \"frontend\"}\n    ...> resource = K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"test\", labels)\n    ...> K8s.Selector.match_labels?(resource, labels)\n    true\n\n  Returns `false` when not matching all labels:\n    iex> not_a_match = %{\"env\" => \"prod\", \"tier\" => \"frontend\", \"nope\" => \"not-a-match\"}\n    ...> resource = K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"test\", %{\"env\" => \"prod\", \"tier\" => \"frontend\"})\n    ...> K8s.Selector.match_labels?(resource, not_a_match)\n    false","ref":"K8s.Selector.html#match_labels?/2-examples"},{"type":"function","title":"K8s.Selector.parse/1","doc":"Parses a `\"selector\"` map of `\"matchLabels\"` and `\"matchExpressions\"`","ref":"K8s.Selector.html#parse/1"},{"type":"function","title":"Examples - K8s.Selector.parse/1","doc":"iex> selector = %{\n    ...>   \"matchLabels\" => %{\"component\" => \"redis\"},\n    ...>   \"matchExpressions\" => [\n    ...>     %{\"operator\" => \"In\", \"key\" => \"tier\", \"values\" => [\"cache\"]},\n    ...>     %{\"operator\" => \"NotIn\", \"key\" => \"environment\", \"values\" => [\"dev\"]}\n    ...>   ]\n    ...> }\n    ...> K8s.Selector.parse(selector)\n    %K8s.Selector{match_labels: %{\"component\" => \"redis\"}, match_expressions: [%{\"operator\" => \"In\", \"key\" => \"tier\", \"values\" => [\"cache\"]},%{\"operator\" => \"NotIn\", \"key\" => \"environment\", \"values\" => [\"dev\"]}]}","ref":"K8s.Selector.html#parse/1-examples"},{"type":"function","title":"K8s.Selector.serialize_match/1","doc":"Returns a `fieldSelector` query string value for a set of field selectors.","ref":"K8s.Selector.html#serialize_match/1"},{"type":"function","title":"Examples - K8s.Selector.serialize_match/1","doc":"Builds a query string for a single field (`kubectl get pods --field-selector status.phase=Running`):\n\n    iex> K8s.Selector.serialize_match(%{\"status.phase\" => {\"=\", \"Running\"}})\n    [\"status.phase=Running\"]\n\n  Builds a query string for multiple fields (`kubectl get pods --field-selector status.phase=Running,metadata.namespace!=default`):\n\n    iex> K8s.Selector.serialize_match(%{\"metadata.namespace\" => {\"!=\", \"default\"}, \"status.phase\" => {\"=\", \"Running\"}})\n    [\"metadata.namespace!=default\", \"status.phase=Running\"]","ref":"K8s.Selector.html#serialize_match/1-examples"},{"type":"function","title":"K8s.Selector.serialize_match_expressions/1","doc":"Returns a `labelSelector` query string value for a set of label expressions.\n\nFor `!=` matches, use a `NotIn` set-based expression.","ref":"K8s.Selector.html#serialize_match_expressions/1"},{"type":"function","title":"Examples - K8s.Selector.serialize_match_expressions/1","doc":"Builds a query string for `In` expressions (`kubectl get pods -l 'environment in (production,qa),tier in (frontend)`):\n\n    iex> expressions = [\n    ...>   %{\"operator\" => \"In\", \"key\" => \"environment\", \"values\" => [\"production\", \"qa\"]},\n    ...>   %{\"operator\" => \"In\", \"key\" => \"tier\", \"values\" => [\"frontend\"]},\n    ...> ]\n    ...> K8s.Selector.serialize_match_expressions(expressions)\n    [\"environment in (production,qa)\", \"tier in (frontend)\"]\n\n  Builds a query string for `NotIn` expressions (`kubectl get pods -l 'environment notin (frontend)`):\n\n    iex> expressions = [\n    ...>   %{\"operator\" => \"NotIn\", \"key\" => \"environment\", \"values\" => [\"frontend\"]}\n    ...> ]\n    ...> K8s.Selector.serialize_match_expressions(expressions)\n    [\"environment notin (frontend)\"]\n\n  Builds a query string for `Exists` expressions (`kubectl get pods -l 'environment'`):\n\n    iex> expressions = [\n    ...>   %{\"operator\" => \"Exists\", \"key\" => \"environment\"}\n    ...> ]\n    ...> K8s.Selector.serialize_match_expressions(expressions)\n    [\"environment\"]\n\n  Builds a query string for `DoesNotExist` expressions (`kubectl get pods -l '!environment'`):\n\n    iex> expressions = [\n    ...>   %{\"operator\" => \"DoesNotExist\", \"key\" => \"environment\"}\n    ...> ]\n    ...> K8s.Selector.serialize_match_expressions(expressions)\n    [\"!environment\"]","ref":"K8s.Selector.html#serialize_match_expressions/1-examples"},{"type":"function","title":"K8s.Selector.serialize_match_labels/1","doc":"","ref":"K8s.Selector.html#serialize_match_labels/1"},{"type":"function","title":"K8s.Selector.to_s/1","doc":"Serializes a `K8s.Selector` to a `labelSelector` query string.","ref":"K8s.Selector.html#to_s/1"},{"type":"function","title":"Examples - K8s.Selector.to_s/1","doc":"iex> selector = K8s.Selector.label({\"component\", \"redis\"})\n  ...> K8s.Selector.to_s(selector)\n  \"component=redis\"","ref":"K8s.Selector.html#to_s/1-examples"},{"type":"type","title":"K8s.Selector.selector_or_operation_t/0","doc":"","ref":"K8s.Selector.html#t:selector_or_operation_t/0"},{"type":"type","title":"K8s.Selector.t/0","doc":"","ref":"K8s.Selector.html#t:t/0"},{"type":"module","title":"K8s.Sys.Event","doc":"Telemetry event defimitions for this library","ref":"K8s.Sys.Event.html"},{"type":"function","title":"K8s.Sys.Event.events/0","doc":"","ref":"K8s.Sys.Event.html#events/0"},{"type":"module","title":"K8s.Sys.Logger","doc":"Attaches telemetry events to the Elixir Logger","ref":"K8s.Sys.Logger.html"},{"type":"function","title":"K8s.Sys.Logger.attach/0","doc":"Attaches telemetry events to the Elixir Logger","ref":"K8s.Sys.Logger.html#attach/0"},{"type":"macro","title":"K8s.Sys.Logger.log_prefix/1","doc":"","ref":"K8s.Sys.Logger.html#log_prefix/1"},{"type":"module","title":"K8s.Sys.Telemetry","doc":"Telemetry event defimitions for this library","ref":"K8s.Sys.Telemetry.html"},{"type":"function","title":"K8s.Sys.Telemetry.events/0","doc":"","ref":"K8s.Sys.Telemetry.html#events/0"},{"type":"module","title":"K8s.Version","doc":"Kubernetes [API Versioning](https://kubernetes.io/docs/concepts/overview/kubernetes-api/#api-versioning)","ref":"K8s.Version.html"},{"type":"function","title":"K8s.Version.alpha/1","doc":"Returns only alpha versions","ref":"K8s.Version.html#alpha/1"},{"type":"function","title":"Examples - K8s.Version.alpha/1","doc":"iex> K8s.Version.alpha([\"v1alpha1\", \"v1\", \"v2beta1\"])\n    [%K8s.Version{major: 1, minor: \"alpha\", patch: 1}]","ref":"K8s.Version.html#alpha/1-examples"},{"type":"function","title":"K8s.Version.beta/1","doc":"Returns only beta versions","ref":"K8s.Version.html#beta/1"},{"type":"function","title":"Examples - K8s.Version.beta/1","doc":"iex> K8s.Version.beta([\"v1alpha1\", \"v1\", \"v2beta1\"])\n    [%K8s.Version{major: 2, minor: \"beta\", patch: 1}]","ref":"K8s.Version.html#beta/1-examples"},{"type":"function","title":"K8s.Version.edge/1","doc":"Retuns all non-stable versions","ref":"K8s.Version.html#edge/1"},{"type":"function","title":"Examples - K8s.Version.edge/1","doc":"iex> K8s.Version.edge([\"v1alpha1\", \"v1\", \"v2beta1\"])\n    [%K8s.Version{major: 1, minor: \"alpha\", patch: 1}, %K8s.Version{major: 2, minor: \"beta\", patch: 1}]","ref":"K8s.Version.html#edge/1-examples"},{"type":"function","title":"K8s.Version.parse/1","doc":"Parses a Kubernetes API version into a `K8s.Version` struct","ref":"K8s.Version.html#parse/1"},{"type":"function","title":"Examples - K8s.Version.parse/1","doc":"iex> K8s.Version.parse(\"v1\")\n    %K8s.Version{major: 1, minor: \"stable\", patch: nil}\n\n    iex> K8s.Version.parse(\"v1beta2\")\n    %K8s.Version{major: 1, minor: \"beta\", patch: 2}\n\n    iex> K8s.Version.parse(\"v1alpha1\")\n    %K8s.Version{major: 1, minor: \"alpha\", patch: 1}","ref":"K8s.Version.html#parse/1-examples"},{"type":"function","title":"K8s.Version.recommended/1","doc":"Returns the recommended version from a list of versions.\n\nIt will return the most stable version if any, otherwise the newest \"*edge*\" version.","ref":"K8s.Version.html#recommended/1"},{"type":"function","title":"Examples - K8s.Version.recommended/1","doc":"iex> K8s.Version.recommended([\"v1beta1\", \"v1\", \"v2\"])\n    \"v2\"\n\n    iex> K8s.Version.recommended([\"v1beta1\", \"v1\"])\n    \"v1\"\n\n    iex> K8s.Version.recommended([\"v1beta2\", \"v1beta1\"])\n    \"v1beta2\"\n\n    iex> K8s.Version.recommended([\"v1alpha1\", \"v1alpha2\", \"v1beta1\"])\n    \"v1beta1\"","ref":"K8s.Version.html#recommended/1-examples"},{"type":"function","title":"K8s.Version.serialize/1","doc":"Returns only stable versions","ref":"K8s.Version.html#serialize/1"},{"type":"function","title":"Examples - K8s.Version.serialize/1","doc":"iex> K8s.Version.serialize(%K8s.Version{major: 1, minor: \"stable\", patch: nil})\n    \"v1\"\n\n    iex> K8s.Version.serialize(%K8s.Version{major: 1, minor: \"beta\", patch: 1})\n    \"v1beta1\"\n\n    iex> K8s.Version.serialize(%K8s.Version{major: 1, minor: \"alpha\", patch: 1})\n    \"v1alpha1\"","ref":"K8s.Version.html#serialize/1-examples"},{"type":"function","title":"K8s.Version.sort/1","doc":"Sorts a list of `K8s.Version`s","ref":"K8s.Version.html#sort/1"},{"type":"function","title":"Examples - K8s.Version.sort/1","doc":"iex> raw_versions = Enum.shuffle([\"v1\", \"v2beta1\", \"v2\", \"v1alpha1\", \"v1alpha2\"])\n    iex> versions = raw_versions |> Enum.map(&(K8s.Version.parse(&1)))\n    iex> sorted_versions = versions |> K8s.Version.sort()\n    iex> sorted_versions |> Enum.map(&(K8s.Version.serialize(&1)))\n    [\"v2\", \"v1\", \"v2beta1\", \"v1alpha2\", \"v1alpha1\"]","ref":"K8s.Version.html#sort/1-examples"},{"type":"function","title":"K8s.Version.stable/1","doc":"Returns only stable versions","ref":"K8s.Version.html#stable/1"},{"type":"function","title":"Examples - K8s.Version.stable/1","doc":"iex> K8s.Version.stable([\"v1alpha1\", \"v1\", \"v2beta1\"])\n    [%K8s.Version{major: 1, minor: \"stable\", patch: nil}]","ref":"K8s.Version.html#stable/1-examples"},{"type":"type","title":"K8s.Version.t/0","doc":"","ref":"K8s.Version.html#t:t/0"},{"type":"exception","title":"K8s.Client.APIError","doc":"Kubernetes API Error\n\nAny HTTP Error with JSON error payload","ref":"K8s.Client.APIError.html"},{"type":"function","title":"K8s.Client.APIError.from_kubernetes_error/1","doc":"","ref":"K8s.Client.APIError.html#from_kubernetes_error/1"},{"type":"type","title":"K8s.Client.APIError.t/0","doc":"","ref":"K8s.Client.APIError.html#t:t/0"},{"type":"exception","title":"K8s.Client.HTTPError","doc":"Kubernetes API Error\n\nAny HTTP Error with JSON error payload","ref":"K8s.Client.HTTPError.html"},{"type":"function","title":"K8s.Client.HTTPError.from_exception/1","doc":"","ref":"K8s.Client.HTTPError.html#from_exception/1"},{"type":"function","title":"K8s.Client.HTTPError.new/1","doc":"","ref":"K8s.Client.HTTPError.html#new/1"},{"type":"type","title":"K8s.Client.HTTPError.t/0","doc":"","ref":"K8s.Client.HTTPError.html#t:t/0"},{"type":"exception","title":"K8s.Conn.Error","doc":"","ref":"K8s.Conn.Error.html"},{"type":"type","title":"K8s.Conn.Error.t/0","doc":"","ref":"K8s.Conn.Error.html#t:t/0"},{"type":"exception","title":"K8s.Discovery.Error","doc":"","ref":"K8s.Discovery.Error.html"},{"type":"type","title":"K8s.Discovery.Error.t/0","doc":"","ref":"K8s.Discovery.Error.html#t:t/0"},{"type":"exception","title":"K8s.Operation.Error","doc":"","ref":"K8s.Operation.Error.html"},{"type":"type","title":"K8s.Operation.Error.t/0","doc":"","ref":"K8s.Operation.Error.html#t:t/0"},{"type":"extras","title":"K8s","doc":"# K8s\n\n[![Module Version](https://img.shields.io/hexpm/v/k8s.svg)](https://hex.pm/packages/k8s)\n[![Coverage Status](https://coveralls.io/repos/github/coryodaniel/k8s/badge.svg?branch=develop)](https://coveralls.io/github/coryodaniel/k8s?branch=develop)\n[![Last Updated](https://img.shields.io/github/last-commit/coryodaniel/k8s.svg)](https://github.com/coryodaniel/k8s/commits/develop)\n\n[![Build Status CI](https://github.com/coryodaniel/k8s/actions/workflows/ci.yaml/badge.svg)](https://github.com/coryodaniel/k8s/actions/workflows/ci.yaml)\n[![Build Status Elixir](https://github.com/coryodaniel/k8s/actions/workflows/elixir_matrix.yaml/badge.svg)](https://github.com/coryodaniel/k8s/actions/workflows/elixir_matrix.yaml)\n[![Build Status K8s](https://github.com/coryodaniel/k8s/actions/workflows/k8s_matrix.yaml/badge.svg)](https://github.com/coryodaniel/k8s/actions/workflows/k8s_matrix.yaml)\n\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/k8s/)\n[![Total Download](https://img.shields.io/hexpm/dt/k8s.svg)](https://hex.pm/packages/k8s)\n[![License](https://img.shields.io/hexpm/l/k8s.svg)](https://github.com/coryodaniel/k8s/blob/develop/LICENSE)\n\n[K8s](https://hexdocs.pm/k8s/usage.html) - Kubernetes API Client for Elixir","ref":"readme.html"},{"type":"extras","title":"Features - K8s","doc":"- A client API for humans 👩🏼🧑👩🏻👩🏽👩🏾🧑🏻🧑🏽🧑🧑🏾👨🏼👨🏾👨🏿\n- 🔮 Kubernetes resources, groups, and CRDs are autodiscovered at boot time. No swagger file to include or override.\n- Client supports standard HTTP calls, async batches, wait on status ⏲️, and watchers 👀\n- ⚙️ HTTP Request middleware\n- Multiple clusters ⚓ ⚓ ⚓\n- 🔐 Multiple authentication credentials\n  - 🤖 serviceaccount\n  - token\n  - 📜 certificate\n  - auth-provider\n  - Pluggable auth providers!\n- 🆗 Tested against Kubernetes versions 1.10+ and master\n- 🛠️ CRD support\n- 📈 Integrated with `:telemetry`\n- ℹ️ Kubernetes resource and version helper functions\n- 🧰 Kube config file parsing\n- 🏎️ Macro free; fast compile & fast startup","ref":"readme.html#features"},{"type":"extras","title":"Installation - K8s","doc":"The package can be installed by adding `:k8s` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:k8s, \"~> 2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - K8s","doc":"Check out the [Usage Guide](https://hexdocs.pm/k8s/usage.html) for in-depth examples. If you like\nlearning with Livebook, check out [kino_k8s](https://github.com/mruoss/kino_k8s). It comes with\nnice smart cells to help you generate your first working code.\n\nMost functions are also written using doctests.\n\n- [K8s.Client doctests](https://hexdocs.pm/k8s/K8s.Client.html)\n- [K8s.Conn doctests](https://hexdocs.pm/k8s/K8s.Conn.html)\n- [K8s.Resource doctests](https://hexdocs.pm/k8s/K8s.Resource.html)\n- [K8s.Version doctests](https://hexdocs.pm/k8s/K8s.Version.html)\n\nIf you are interested in building Kubernetes Operators or Schedulers, check out [Bonny](https://github.com/coryodaniel/bonny).","ref":"readme.html#usage"},{"type":"extras","title":"tl;dr Examples - K8s","doc":"#","ref":"readme.html#tl-dr-examples"},{"type":"extras","title":"Configure a cluster connection - K8s","doc":"Cluster connections can be created using the `K8s.Conn` module.\n\n`K8s.Conn.from_file/1` will use the current context in your kubeconfig.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n```\n\n`K8s.Conn.from_file/2` accepts a keyword list to set the `:user`, `:cluster`, and/or `:context`\n\nConnections can also be created in-cluster from a service account.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_service_account(\"/path/to/service-account/directory\")\n```\n\nCheck out the [connection guide](https://hexdocs.pm/k8s/connections.html) for additional details.\n\n#","ref":"readme.html#configure-a-cluster-connection"},{"type":"extras","title":"Creating a deployment - K8s","doc":"```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\nopts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\n{:ok, resource} = K8s.Resource.from_file(\"priv/deployment.yaml\", opts)\n\noperation = K8s.Client.create(resource)\n{:ok, deployment} = K8s.Client.run(conn, operation)\n```\n\n#","ref":"readme.html#creating-a-deployment"},{"type":"extras","title":"Listing deployments - K8s","doc":"In a namespace:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\noperation = K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: \"prod\")\n{:ok, deployments} = K8s.Client.run(conn, operation)\n```\n\nAcross all namespaces:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\noperation = K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: :all)\n{:ok, deployments} = K8s.Client.run(conn, operation)\n```\n\n#","ref":"readme.html#listing-deployments"},{"type":"extras","title":"Getting a deployment - K8s","doc":"```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\noperation = K8s.Client.get(\"apps/v1\", :deployment, [namespace: \"default\", name: \"nginx-deployment\"])\n{:ok, deployment} = K8s.Client.run(conn, operation)\n```","ref":"readme.html#getting-a-deployment"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!-- #","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!--------------------- Don't add new entries after this line --------------------->\n## [2.5.0] - 2023-11-12\n\n#","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Conn` - Add namespace info in case of service account connection. [#291](https://github.com/coryodaniel/k8s/pull/291)\n\n## [2.4.2] - 2023-10-18\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.Mint.HTTPAdapter` - Only stop the process if the connection is closed for reading (and writing). - [#280](https://github.com/coryodaniel/k8s/issues/280), [#285](https://github.com/coryodaniel/k8s/pull/285)\n\n## [2.4.1] - 2023-08-15\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.Mint.HTTPAdapter` - Don't raise exception when terminating an adapter. - [#268](https://github.com/coryodaniel/k8s/issues/268), [#269](https://github.com/coryodaniel/k8s/pull/269)\n\n## [2.4.0] - 2023-07-07\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Client.wait_until/2` - Allow passing DELETE operations in order to wait for deletion. - [#262](https://github.com/coryodaniel/k8s/pull/262)\n\n## [2.3.0] - 2023-05-14\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Client.connect/4` - Support connecting to `pods/log` subresource. - [#254](https://github.com/coryodaniel/k8s/issues/254), [#255](https://github.com/coryodaniel/k8s/issues/255)\n- `K8s.Conn.from_env/2` - Generates configuration from a file defined by an env variable. - [#251](https://github.com/coryodaniel/k8s/pull/251)\n- `K8s.Conn` - Better hexdocs\n\n## [2.2.0] - 2023-03-27\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Conn.Auth.Exec` - Define default value for `:args` - [#240](https://github.com/coryodaniel/k8s/pull/240)\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Conn.Auth.Azure` - Azure auth provider added by @hanspagh - [#162](https://github.com/coryodaniel/k8s/issues/162), [#225](https://github.com/coryodaniel/k8s/issues/225)\n\n## [2.1.1] - 2023-03-02\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Watcher reset resource version for objects with a \"message\" field. - [#232](https://github.com/coryodaniel/k8s/issues/232), [#231](https://github.com/coryodaniel/k8s/issues/231)\n\n## [2.1.0] - 2023-02-25\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added further PATCH mechanisms - [#229](https://github.com/coryodaniel/k8s/pull/229)\n- Add `opts` to `K8s.Conn.from_file/N` and `K8s.Conn.from_service_account/N` in order to be able to pass `:insecure_skip_tls_verify` option directly. - [#230](https://github.com/coryodaniel/k8s/issues/230), [#203](https://github.com/coryodaniel/k8s/issues/203)\n\n## [2.0.3] - 2023-02-17\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- A regression introduced in 2.0.2: superfluous call to `Genserver.reply()` was removed.\n\n## [2.0.2] - 2023-02-16\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Resume watch when API server has gone away - [#226](https://github.com/coryodaniel/k8s/pull/226), [#222](https://github.com/coryodaniel/k8s/issues/222)\n\n## [2.0.1] - 2023-02-12\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Requests with large bodies are failing - [#221](https://github.com/coryodaniel/k8s/pull/221), [#220](https://github.com/coryodaniel/k8s/issues/220)\n\n## [2.0.0] - 2023-01-27\n\nThis version comes with some breaking changes. Please refer to the\n[migrations guide](./guides/migrations.md) for help on how to migrate your\nprojects to this version.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Selector.label_not/N`, `K8s.Selector.field/N` and `K8s.Selector.field_not/N` - Support for field selectors ([#117](https://github.com/coryodaniel/k8s/pull/117))\n- `K8s.Client.Provider.stream/5` callback was added to the behaviour\n- `K8s.Client.Runner.Base.stream/3`\n- `K8s.Client.Provider.stream_to/6` callback was added to the behaviour\n- `K8s.Client.Runner.Base.stream_to/4`\n- `K8s.Client.MintHTTPProvider` - The mint client implementation\n- `K8s.Client.HTTPTestHelper` - to be used in tests (resides in `lib/` so it can be used by dependents)\n- Open `:connect` operations (connections) now accept messages to be sent to pods if using `K8s.Client.stream_to/N`\n- `K8s.Client.put_conn/2` to add pielining support to the Client API\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `K8s.Client.Provider` behaviour was adapted to the new internal architecture\n- `K8s.Client.watch/N` now returns a `:watch` or `:watch_all_namespaces` operation to be passed to `K8s.Client.stream/N`\n- `Websockex` was replaced by [`Mint.WebSocket`](https://hexdocs.pm/mint_web_socket/Mint.WebSocket.html)\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- `K8s.Client.HTTPProvider` was removed in favor of `K8s.Client.MintHTTPProvider`\n- The `:stream_to` in `http_opts` was removed in favor of `K8s.Client.stream_to/N` and `K8s.Client.stream/N`.\n- `K8s.Client.DynamicWebSocketProvider` was removed. Use `K8s.Client.DynamcHTTPProvider.websocket*` functions instead .\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Tests using the `DynamicHTTPProvider` which work with `watch_and_stream` are going to need to be changed. The HTTP mocks now need to implement the `stream/5` callback. (See `K8s.Client.Runner.Watch.StreamTest` on this branch for examples)d.\n- `K8s.Client.DynamicWebSocketProvider` was removed in favor of `K8s.Client.DynamcHTTPProvider.websocket*` functions.\n- The `:stream_to` in `http_opts` is not supported anymore. Use `K8s.Client.stream/N` and `K8s.Client.stream_to/N` instead.\n- Errors are encapsulated in `K8s.Client.HTTPError`\n- `headers/1` callback was removed from `K8s.Client.Provider` behaviour.\n- `K8s.Client.HTTPProvider` (HTTPoison implementation) was removed.\n- `K8s.Client.watch/N` now returns a `:watch` or `:watch_all_namespaces` operation to be passed to `K8s.Client.stream/N`\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Fixed - Changelog","doc":"- Update `PKI.cert_from_map/2` to support fully qualified domain names (FQDN) - Fix for `K8s.Conn.from_file/1` ([#164](https://github.com/coryodaniel/k8s/pull/164))\n\n## [2.0.0-rc.6] - 2023-01-19\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Unable to parse response (invalid JSON) ([#215](https://github.com/coryodaniel/k8s/pull/215))\n\n## [2.0.0-rc.5] - 2023-01-08\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- `K8s.Client.Mint.HTTPAdapter` - Handle Elixir streams separately from `stream_to`.\n\n## [2.0.0-rc.4] - 2023-01-07\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `:poolboy` - Pooling for HTTP/1 connections\n- `K8s.Client.Mint.HTTPAdapter` - Monitor caller and cleanup state upon `:DOWN`\n\n## [2.0.0-rc.3] - 2023-01-01\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.Mint.ConnectionRegistry` - closed connections were not re-established.\n- `K8s.Client.Mint.Request.HTTP` - Add missing struct field `:waiting`\n\n## [2.0.0-rc.2] - 2022-12-31\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Client.Mint.HTTPAdapter` - A GenServer handling `Mint.HTTP` connections.\n- `K8s.Client.Mint.ConnectionRegistry` - The registry for open `HTTP/2` connections\n\n## [2.0.0-rc.1] - 2022-12-19\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.Mint.WebSocket` - Close websocket if process is terminated\n\n## [2.0.0-rc.0] - 2022-12-14\n\nThis version comes with some breaking changes. Please refer to the\n[migrations guide](./guides/migrations.md) for help on how to migrate your\nprojects to this version.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Selector.label_not/N`, `K8s.Selector.field/N` and `K8s.Selector.field_not/N` - Support for field selectors ([#117](https://github.com/coryodaniel/k8s/pull/117))\n- `K8s.Client.Provider.stream/5` callback was added to the behaviour\n- `K8s.Client.Runner.Base.stream/3`\n- `K8s.Client.Provider.stream_to/6` callback was added to the behaviour\n- `K8s.Client.Runner.Base.stream_to/4`\n- `K8s.Client.MintHTTPProvider` - The mint client implementation\n- `K8s.Client.HTTPTestHelper` - to be used in tests (resides in `lib/` so it can be used by dependents)\n- Open `:connect` operations (connections) now accept messages to be sent to pods\n- `K8s.Client.put_conn/2` to add pielining support to the Client API\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `K8s.Client.Provider` behaviour was adapted to the new internal architecture\n- `K8s.Client.watch/N` now returns a `:watch` or `:watch_all_namespaces` operation to be passed to `K8s.Client.stream/N`\n- `Websockex` was replaced by [`Mint.WebSocket`](https://hexdocs.pm/mint_web_socket/Mint.WebSocket.html)\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- `K8s.Client.HTTPProvider` was removed in favor of `K8s.Client.MintHTTPProvider`\n- The `:stream_to` in `http_opts` was removed in favor of `K8s.Client.stream_to/N` and `K8s.Client.stream/N`.\n- `K8s.Client.DynamicWebSocketProvider` was removed. Use `K8s.Client.DynamcHTTPProvider.websocket*` functions instead .\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Tests using the `DynamicHTTPProvider` which work with `watch_and_stream` are going to need to be changed. The HTTP mocks now need to implement the `stream/5` callback. (See `K8s.Client.Runner.Watch.StreamTest` on this branch for examples)d.\n- `K8s.Client.DynamicWebSocketProvider` was removed in favor of `K8s.Client.DynamcHTTPProvider.websocket*` functions.\n- The `:stream_to` in `http_opts` is not supported anymore. Use `K8s.Client.stream/N` and `K8s.Client.stream_to/N` instead.\n- Errors are encapsulated in `K8s.Client.HTTPError`\n- `headers/1` callback was removed from `K8s.Client.Provider` behaviour.\n- `K8s.Client.HTTPProvider` (HTTPoison implementation) was removed.\n- `K8s.Client.watch/N` now returns a `:watch` or `:watch_all_namespaces` operation to be passed to `K8s.Client.stream/N`\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Fixed - Changelog","doc":"- Update `PKI.cert_from_map/2` to support fully qualified domain names (FQDN) - Fix for `K8s.Conn.from_file/1` ([#164](https://github.com/coryodaniel/k8s/pull/164))\n\n## [1.2.0] - 2022-12-07\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Selector.label_not/N`, `K8s.Selector.field/N` and `K8s.Selector.field_not/N` - Support for field selectors ([#117](https://github.com/coryodaniel/k8s/pull/117))\n- `K8s.Client.connect/3` - Executes a [command in a Pod](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.22/#execaction-v1-core) [#190](https://github.com/coryodaniel/k8s/pull/190)\n\n## [1.1.10] - 2022-10-30\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.run/2`: spec updated to include `t:K8s.Discovery.Error.t/0` in possible error structs\n\n## [1.1.9] - 2022-10-28\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.run/2`: spec updated to include `t:K8s.Client.APIError.t/0` in possible error structs ([#189](https://github.com/coryodaniel/k8s/pull/189))\n- `K8s.Operation.Path.build/1`: Allow namespace to be `nil`\n\n## [1.1.8] - 2022-10-26\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"`K8s.Client.watch_and_stream/2`: Get resource vesrion before creating the Stream resource in order to not miss anything. ([#187](https://github.com/coryodaniel/k8s/pull/187))\n\n## [1.1.7] - 2022-10-15\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix protocol implementations in Elixir 1.14.1: Replace `__MODULE__` with actual module name. [#185](https://github.com/coryodaniel/k8s/pull/185)\n- Match subresources when the kind does not equal the subresource ([#184](https://github.com/coryodaniel/bonny/issues/184))\n\n## [1.1.6] - 2022-10-03\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Resourse.label/2`: spec updated to accept label maps as a second argument [#177](https://github.com/coryodaniel/k8s/pull/177)\n- `K8s.Discovery.Driver.File`: Use `conn.discovery_opts` in file discovery driver ([#180](https://github.com/coryodaniel/k8s/pull/180))\n- `K8s.Client.DynamicHTTPProvider.request/5`: Fix converting PID to string inside error message ([#181](https://github.com/coryodaniel/k8s/pull/181))\n\n## [1.1.5] - 2022-05-19\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- `K8s.Client.watch_and_stream/2`: 410 Gone not rescued [#159](https://github.com/coryodaniel/k8s/pull/159)\n- `K8s.Client.watch/3`: `get` operations should be transformed to `list` BEFORE retrieving the resource version [#160](https://github.com/coryodaniel/k8s/pull/160)\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- `K8s.Client.watch_and_stream/2`: Request BOOKMARK events and process them when watching resource collections. [#159](https://github.com/coryodaniel/k8s/pull/159)\n\n## [1.1.4] - 2022-03-15\n\n- Fix more authorization headers that are not keyword lists [#148](https://github.com/coryodaniel/k8s/pull/148)\n\n## [1.1.3] - 2022-03-13\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix default value in `K8s.Client.Runner.Watch.run/4` and `K8s.Client.Runner.Watch.stream/3`\n\n## [1.1.2] - 2022-03-13\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Support for FQDN K8s API servers and Root CA chains [#144](https://github.com/coryodaniel/k8s/issues/144)\n- Wrong exception raised by `K8s.Resource.from_file!/2` [#137](https://github.com/coryodaniel/k8s/issues/137), [#143](https://github.com/coryodaniel/k8s/issues/143)\n- `K8s.Client.Runner.Watch.stream/3` - convert `:get` to `:list` operation with field selector.\n- Make Logger metadata `library: :k8s` available at compile time.\n\n## [1.1.1] - 2022-03-01\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Initialize authorization header as valid keyword list [#142](https://github.com/coryodaniel/k8s/pull/142)\n- Restore deprecated `K8s.Sys.Event` module.\n\n## [1.1.0] - 2022-02-21\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Client.Runner.Watch.stream/3` - watches a resource and returns an elixir [Stream](https://hexdocs.pm/elixir/1.12/Stream.html) of events #121\n- `K8s.Client.apply/2` - Create a [server-side apply](https://kubernetes.io/docs/reference/using-api/server-side-apply/) operation\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Handle generic kubernetes response Failure without a reason [#120](https://github.com/coryodaniel/k8s/pull/120)\n- Replace Notion with Telemetry and improve Logging [#128](https://github.com/coryodaniel/k8s/pull/128)\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- `K8s.Client.HTTPProvider.headers/2` was deprecated in favor of `K8s.Client.HTTPProvider.headers/1`\n- `K8s.Client.DynamicHTTPProvider.headers/2` was deprecated in favor of `K8s.Client.DynamicHTTPProvider.headers/1`\n\n#","ref":"changelog.html#deprecated"},{"type":"extras","title":"Fixed - Changelog","doc":"- Preserve namespace in `get_to_list/1` [#122](https://github.com/coryodaniel/k8s/issues/122), [#123](https://github.com/coryodaniel/k8s/pull/123)\n- Fix obsolete doc on wait operation [#118](https://github.com/coryodaniel/k8s/pull/118)\n- Dialyzer errors with K8s.Client functions [#119](https://github.com/coryodaniel/k8s/issues/119)\n- Enable peer certificate authentication #127. Be aware, this will break configurations that have been using incorrect certificate(s) up to this point.\n\n## [1.0.0] - 2021-07-19\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Resource.NamedList.access!/1` raises if item is missing\n- K8s.Operation.put_label_selector/2\n- K8s.Operation.get_label_selector/1\n- Per connection http provider configuration\n- K8s.Operation now uses keyword lists for query_params instead of maps\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- error tuples refactored away from binary and atom to exception modules\n- removed dialyzer exceptions\n- `K8s.Conn.from_file/2` now returns an ok or error tuple\n- `K8s.Conn.from_service_account/N` now returns an ok or error tuple\n- `K8s.Conn.t()` is now the first argument in all runners. `K8s.Operation.t()` is now the second.\n- deprecated K8s.http_provider/0\n- deprecated K8s.Discovery.default_opts/0\n- deprecated K8s.Discovery.default_driver/0\n- Refactored cluster names to strings\n- `K8s.Resource.NamedList.access/1` deals better with missing items now\n- Removed K8s.Client.run/4, use `K8s.Client.run/3` to pass options to HTTP provider\n- Middleware moved to K8s.Conn.Middleware\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- K8s.Conn.lookup/1\n- config.exs based cluster registration is no longer supported, build K8s.Conn using K8s.Conn module\n- environment variable based cluster registration has been removed and may be moved to an external library\n\n## [0.5.2] - 2020-07-31\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Added auth `exec` support\n\n## [0.5.1] - 2020-07-17\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- K8s.Operation struct `query_params` field\n- BasicAuth auth provider\n- Deprecated HTTPoison options being passed to K8s.Client.Runner.base\n- K8s.Operation.put_query_param/3 to add query parameters by key\n- K8s.Operation.get_query_param/3 to get a query parameter by key\n- DigitalOcean authentication\n- `K8s.Resource.NamedList.access/1` - Accessor for lists with named items (containers, env, ...) ([#82](https://github.com/coryodaniel/k8s/pull/82))\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Refactored old references to `cluster_name` to `conn`\n\n## [0.5.0] - 2020-02-12\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- #42 Request middleware support\n- #43 Just in time discovery: K8s.Discovery\n- #44 Support for ad-hoc connections. K8s.Conn based functions. Build your own Conn at runtime or config mix/env vars. No more Cluster registry.\n- K8s.Resource.from_file/2 and K8s.Resource.all_from_file/2 - non-exception versions\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Boot time discovery K8s.Cluster.Discovery\n- K8s.Cluster.base_url/1\n- Remove K8s.Cluster\\*\n\n## [0.4.0] - 2019-08-29\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Changed - Changelog","doc":"- Renamed `K8s.Conf` to `K8s.Conn`\n- Refactored `:conf` configuration key to `:conn`\n\n## [0.3.2] - 2019-08-15\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Selector.match_expressions?/2` to check if a resource matches expressions\n- `K8s.Selector.match_labels?/2` to check if a resource matches labels\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `K8s.Resource` functions moved to submodule\n\n## [0.3.1] - 2019-08-15\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- text/plain response handling\n- K8s.Selector - labelSelector support for K8s.Operation\n\n## [0.3.0] - 2019-07-29\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `K8s.Client.DynamicHTTPProvider` to allow per-process registering of HTTP request handlers.\n- `K8s.Cluster.Discovery` discovery interface\n- `K8s.Cluster.Discovery.api_versions/1` - queries a cluster for all apiVersions\n- `K8s.Cluster.Discovery.resource_definitions/1` - queries a cluster for all resource definitions\n- `K8s.Cluster.Discovery.HTTPDriver` for discovery via k8s REST API\n- `K8s.Cluster.Discovery.FileDriver` for discovery via a file, used for testing, shipped to help dependent libraries mock discovery\n- Support for creating subresources\n- Support for getting subresources\n- Support for updating subresources\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Refactored tests on DynamicHTTPProvider\n- Refactored discovery to use `K8s.Cluster.Discovery`\n- Set correct content-type for patch operations (https://github.com/coryodaniel/k8s/issues/32)\n- Refactored Operation.kind -> Operation.name\n- Group.cluster_key/2 -> Group.lookup_key/2\n- K8s.Cluster.Group :ets data structure changed to map\n- K8s.Cluster.Group module encompases access to :ets table\n- Refactored Operation.resource -> Operation.data. The term `resource` is a bit overloaded in this repo, since the operation is encapsulating the HTTP request, `data` feels a bit more clear.\n- Refactored internal references to \"group version\" to \"api version\"\n\n## [0.2.13] - 2019-06-27\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- K8s.Cluster.base_url/1\n\n## [0.2.12] - 2019-06-26\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- First K8s.Client.Runner.Stream evaluation made lazy\n- K8s.Resource.api_version/1\n\n## [0.2.11] - 2019-06-24\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- K8s.Resource.cpu/1 parses cpu resource requests/limits strings to number\n- K8s.Resource.memory/1 parses cpu resource requests/limits strings to number\n\n## [0.2.10] - 2019-06-24\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- K8s.Client.Runner.Stream for producing elixir streams from k8s list results\n- K8s 1.15 swagger file\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Reversed pattern matching in functions from `var=pattern` to `pattern=var`\n- Added make target for fetching master swagger before running tests\n\n## [0.2.9] - 2019-06-10\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Kubernetes resources, groups, and CRDs are autodiscovered at boot time. No swagger file to include or override.\n- Client supports standard HTTP calls, async batches, wait on status, and watchers\n- Supports multiple clusters\n- Supports multiple kubernetes APIs in the same runtime\n- serviceaccount authentication\n- token authentication\n- certificate authentication\n- auth-provider authetnicati\n- Tested against kubernetes swagger specs: 1.10+ and master\n- CRD support\n- Kubernetes resource and version helper functions\n- Kube config file parsing\n- Pluggable auth providers","ref":"changelog.html#added"},{"type":"extras","title":"Advanced","doc":"# Advanced","ref":"advanced.html"},{"type":"extras","title":"Custom Resource Definitions - Advanced","doc":"Custom resources are discovered via the same mechanism as \"standard\" k8s resources and can be worked with as such:\n\nListing the `Greeting`s from the [`hello operator`](https://github.com/coryodaniel/hello_operator).\n\n```elixir\noperation = K8s.Client.list(\"hello-operator.example.com/v1\", :greeting, [namespace: \"default\"])\n{:ok, greeting} = K8s.Client.run(operation, :dev)\n```","ref":"advanced.html#custom-resource-definitions"},{"type":"extras","title":"Multiple Clusters - Advanced","doc":"Copying a workloads between two clusters:\n\nRegister a staging cluster:\n\n```elixir\n{:ok, staging_conn} = K8s.Conn.from_file(\"~/.kube/config\")\n```\n\nRegister a prod cluster:\n\n```elixir\n{:ok, prod_conn} = K8s.Conn.from_service_account() # or from_file/2\n```\n\nGet a list of all deployments in the `default` prod namespace:\n\n```elixir\n{:ok, prod_conn} = K8s.Conn.from_service_account() # or from_file/2\noperation = K8s.Client.list(\"apps/v1\", :deployment, namespace: \"default\")\n{:ok, deployments} = K8s.Client.run(prod_conn, operation)\n```\n\nMap the deployments to operations and async create on staging:\n\n```elixir\n{:ok, staging_conn} = K8s.Conn.from_service_account() # or from_file/2\noperations = Enum.map(deployments, fn(deployment) -> K8s.Client.create(deployment) end)\n\nK8s.Client.async(staging_conn, operations)\n```","ref":"advanced.html#multiple-clusters"},{"type":"extras","title":"Performing sub-resource operations - Advanced","doc":"Subresource (eviction|finalize|bindings|binding|approval|scale|status) operations are created in the same way as standard operations using `K8s.Operation.build/4`, `K8s.Operation.build/5`, or any `K8s.Client` function.\n\nGetting a deployment's status:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\noperation = K8s.Client.get(\"apps/v1\", \"deployments/status\", name: \"nginx\", namespace: \"default\")\n{:ok, scale} = K8s.Client.run(conn, operation)\n```\n\nGetting a deployment's scale:\n\n```\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\noperation = K8s.Client.get(\"apps/v1\", \"deployments/scale\", [name: \"nginx\", namespace: \"default\"])\n{:ok, scale} = K8s.Client.run(conn, operation)\n```\n\nThere are two forms for mutating subresources.\n\nEvicting a pod with a Pod map:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\neviction = %{\n  \"apiVersion\" => \"policy/v1beta1\",\n  \"kind\" => \"Eviction\",\n  \"metadata\" => %{\n    \"name\" => \"nginx\",\n  }\n}\n\n# Here we use K8s.Resource.build/4 but this k8s resource map could be built manually or retrieved from the k8s API\nsubject = K8s.Resource.build(\"v1\", \"Pod\", \"default\", \"nginx\")\noperation = K8s.Client.create(subject, eviction)\n{:ok, resp} = K8s.Client.run(conn, operation)\n```\n\nEvicting a pod by providing details:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\neviction = %{\n  \"apiVersion\" => \"policy/v1beta1\",\n  \"kind\" => \"Eviction\",\n  \"metadata\" => %{\n    \"name\" => \"nginx\",\n  }\n}\n\nsubject = K8s.Client.create(\"v1\", \"pods/eviction\", [namespace: \"default\", name: \"nginx\"], eviction)\noperation = K8s.Client.create(subject, eviction)\n{:ok, resp} = K8s.Client.run(conn, operation)\n```","ref":"advanced.html#performing-sub-resource-operations"},{"type":"extras","title":"Authentication (K8s.Conn.Auth)","doc":"# Authentication (`K8s.Conn.Auth`)\n\n`k8s` features pluggable authentication, but includes 5 strategies in the order of attempted application:\n\n* `K8s.Conn.Auth.Certificate` certificate based authentication\n* `K8s.Conn.Auth.Token` token based authentication\n* `K8s.Conn.Auth.AuthProvider` implements a Kubernetes config file's [`auth-provider`](https://banzaicloud.com/blog/kubeconfig-security/) functionality.\n* `K8s.Conn.Auth.Exec` implements a Kubernetes config file's [`exec`](https://banzaicloud.com/blog/kubeconfig-security/)\n functionality.\n* `K8s.Conn.Auth.BasicAuth` username/password basic auth\n\n**A few notes first:**\n\n1. [`K8s.Conn.Auth.AuthProvider`](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn/auth/auth_provider.ex) is itself an authentication strategy that allow shell calls to provide a Bearer Token. It's unfortunately named, but the names of the modules follow the key names in a Kubernetes config file. More on this strategy can be found [here](https://banzaicloud.com/blog/kubeconfig-security/).\n   \n2. The [`K8s.Conn`](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn.ex#L58) struct encapsulates a connection to a cluster. It has the cluster address as well as how to authenticate to the cluster. `K8s.Conn` structs can be constructed manually, but there are a few helpers here to create one.\n   \n3. The [`K8s.Conn.Auth.Token`](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn/auth/token.ex#L13-L14) auth strategy is probably the simplest strategy to review as a reference implementation.","ref":"authentication.html"},{"type":"extras","title":"Custom Authentication Providers - Authentication (K8s.Conn.Auth)","doc":"Two things are required to implement a custom auth strategy:\n\n1. Implement the [K8s.Conn.Auth](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn/auth.ex) **behaviour** for auth strategies. The first strategy to return an `{:ok, K8s.Conn.Auth}` struct will be chosen. Any that cannot authenticate the connection should return `:skip`.\n   \n2. Implement the [K8s.Conn.RequestOptions](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn/request_options.ex#L19) protocol which should create a `RequestOptions` struct. This struct is used to set HTTP Headers and SSL connection options.\n\nLooking at the [Token](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn/auth/token.ex#L13-L14) example:\n\n* Line 13 implements the case where this auth strategy would be able to generate request options\n* Line 14 implements the default case where it cannot authenticate the request\n* Lines 19-24 implement how to generate HTTP Headers and SSL options to be used by HTTPoison to make the HTTP requests.","ref":"authentication.html#custom-authentication-providers"},{"type":"extras","title":"Using a Custom Authentication Provider - Authentication (K8s.Conn.Auth)","doc":"Authentication providers are traversed in order. The first provider to return an `K8s.Conn.Auth` struct is used. Default providers are checked _after_ any providers supplied to in the Mix config key `:auth_providers`:\n\n```elixir\nconfig :k8s, \n  auth_providers: [CustomProvider1, CustomProvider2]\n```\n\nThis would result in authentication attempts in the following order:\n\n1. `CustomProvider1`\n2. `CustomProvider2`\n3. `K8s.Conn.Auth.Certificate`\n4. `K8s.Conn.Auth.Token`\n5. `K8s.Conn.Auth.AuthProvider`\n6. `K8s.Conn.Auth.Exec`\n7. `K8s.Conn.Auth.BasicAuth`\n\nFor protocol and behavior implementation examples check out `K8s.Conn.Auth` implementations [here](https://github.com/coryodaniel/k8s/blob/develop/lib/k8s/conn/auth/).","ref":"authentication.html#using-a-custom-authentication-provider"},{"type":"extras","title":"Connections (K8s.Conn)","doc":"# Connections (`K8s.Conn`)\n\nA connection (`K8s.Conn`) is the encapsulation of cluster information and authentication. They can be built using two helper functions (`K8s.Conn.from_file/2`, `K8s.Conn.from_service_account/2`) or by programmatically.","ref":"connections.html"},{"type":"extras","title":"Building Connections - Connections (K8s.Conn)","doc":"There are a few helper functions for creating `K8s.Conn`s, but they can also be created by using a struct.\n\n**Using `K8s.Conn.from_env/2`:**\n\n`K8s.Conn.from_env/2` takes an env variable name as binary (defaults to `KUBECONFIG`). Bonny reads its value and forwards the call to `K8s.Conn.from_file/2`.\nThe options passed as second argument are the same as for `K8s.Conn.from_file/2`.\n\n```elixir\n# Reads KUBECONFIG env varable:\n{:ok, conn} = K8s.Conn.from_env()\n```\n\nTo pass the env variable explicitely and specify options:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_env(\"K8S_CONFIG_FILE\", insecure_skip_tls_verify: true)\n```\n\n**Using `K8s.Conn.from_file/2`:**\n\n`K8s.Conn.from_file/2` accepts a keyword list as the second argument for selecting a `cluster`, `user`, and/or `context` from your kubeconfig file.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"/path/to/kube/config\", context: \"your-context-name-here\")\n```\n\n**Using `K8s.Conn.from_service_account/2`:**\n\nIf a path isn't provided for the service account, the default path is used `/var/run/secrets/kubernetes.io/serviceaccount`.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_service_account()\n```\n\nOptionally the path can be specified:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_service_account(\"/path/to/service/account/directory\")\n```\n\n**Creating a `K8s.Conn` struct programmatically:**\n\nFor the use case of having an unbound number of connections (a multi-tenant K8s service) connections can be manually created.\n\n```elixir\n{:ok, cert} = K8s.Conn.PKI.cert_from_map(cluster, base_path)\n %K8s.Conn{\n    url: \"https://ip-address-of-cluster\",\n    ca_cert: cert,\n    auth: %K8s.Conn.Auth{},\n    insecure_skip_tls_verify: false,\n    discovery_driver: K8s.Discovery.Driver.HTTP,\n    discovery_opts: [cache: true]\n  }\n```\n\nSee [authentication](./authentication.md) and [discovery](./discovery.md) for details on these options.","ref":"connections.html#building-connections"},{"type":"extras","title":"Discovery (K8s.Discovery)","doc":"# Discovery (`K8s.Discovery`)\n\n`k8s` supports _just-in-time_ discovery of cluster APIs and resource kinds. This is important because depending on the CRDs, Kubernetes version, and enabled APIs, URL paths can vary.\n\nThe goal of the `k8s` library was to support multiple clusters without the need to generate code from Kubernetes swagger files. This makes the library more portable and easier to use with various clusters.\n\n`K8s.Discovery` is pluggable. It was designed in this manner to ease testing. The `K8s.Discovery.Driver.HTTP` driver is _probably_ the only driver you will need to use in development and production.\n\nA file driver (`K8s.Discovery.Driver.File`) is also provided. This is used in the test suite to \"stub\" discovery without making HTTP requests. The file driver could also be used in the scenario that you want to limit calls to the Kubernetes masters either out to reduce the number of HTTP calls or for security if the master's API metadata is inaccessible (`kubectl api-versions`, `kubectl api-resources`).","ref":"discovery.html"},{"type":"extras","title":"Using the Discovery module - Discovery (K8s.Discovery)","doc":"Discovery is done automatically during HTTP requests, but the module can also be used directly.\n\nAvailable functions are:\n\n* `K8s.Discovery.versions/2` - list API versions of the cluster\n* `K8s.Discovery.resources/3` - list resource kinds of a specific API\n* `K8s.Discovery.url_for/2` - Generate the URL for a `K8s.Operation` on a specific `K8s.Conn`","ref":"discovery.html#using-the-discovery-module"},{"type":"extras","title":"Setting the Default Driver - Discovery (K8s.Discovery)","doc":"Drivers can be set via a Mix config. A default driver can be specified as well as a driver per registered connection.\n\n`discovery_driver` and `discovery_opts` control the default driver to use. If not set, the HTTP driver will be used with an empty keyword list.\n\nBelow the file driver is set as the default using `\"test/support/discovery/example.json\"` as the configuration file (see _Using the File Driver_ below).\n\n```elixir\nuse Mix.Config\n\nconfig :k8s,\n  discovery_driver: K8s.Discovery.Driver.File,\n  discovery_opts: [config: \"test/support/discovery/example.json\"]\n```","ref":"discovery.html#setting-the-default-driver"},{"type":"extras","title":"Setting Drivers per Connection - Discovery (K8s.Discovery)","doc":"Drivers can also be set per connection.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"....\")\nconn = %K8s.Conn{conn | discovery_driver: AlternateDriver}\n```","ref":"discovery.html#setting-drivers-per-connection"},{"type":"extras","title":"Using the HTTP Driver (`K8s.Discovery.Driver.HTTP`) - Discovery (K8s.Discovery)","doc":"The HTTP driver is used by default. When a `K8s.Operation` is run calls to `/api` and `/apis` are made to discovery what resources, versions, and scope are supported for resources.\n\nThe HTTP driver can also be used directly to explore API support.\n\nList supported versions:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\nK8s.Discovery.Driver.HTTP.versions(conn)\n[\"v1\", \"apps/v1\", \"batch/v1\"]\n```\n\nList supported resources of an API version.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\nK8s.Discovery.Driver.HTTP.resources(\"apps/v1\", conn)\n[\n  {\n    \"kind\": \"DaemonSet\",\n    \"name\": \"daemonsets\",\n    \"namespaced\": true,\n    \"verbs\": [\n      \"create\",\n      \"delete\",\n      \"deletecollection\",\n      \"get\",\n      \"list\",\n      \"patch\",\n      \"update\",\n      \"watch\"\n    ]\n  },\n  {\n    \"kind\": \"Deployment\",\n    \"name\": \"deployments\",\n    \"namespaced\": true,\n    \"verbs\": [\n      \"create\",\n      \"delete\",\n      \"deletecollection\",\n      \"get\",\n      \"list\",\n      \"patch\",\n      \"update\",\n      \"watch\"\n    ]\n  }\n  # ...\n]\n```","ref":"discovery.html#using-the-http-driver-k8s-discovery-driver-http"},{"type":"extras","title":"Using the File Driver (`K8s.Discovery.Driver.File`) - Discovery (K8s.Discovery)","doc":"The file driver is primarily intended for testing. The format for a driver configuration file is a JSON file with API version as the key to a list of resource metadata.\n\n```json\n{\n  \"v1\": [\n    {\n      \"kind\": \"Namespace\",\n      \"name\": \"namespaces\",\n      \"namespaced\": false,\n      \"verbs\": [\n        \"create\",\n        \"delete\",\n        \"get\",\n        \"list\",\n        \"patch\",\n        \"update\",\n        \"watch\"\n      ]\n    }\n  ],\n  \"apps/v1\": [\n    {\n      \"kind\": \"Deployment\",\n      \"name\": \"deployments\",\n      \"namespaced\": true,\n      \"verbs\": [\n        \"create\",\n        \"delete\",\n        \"deletecollection\",\n        \"get\",\n        \"list\",\n        \"patch\",\n        \"update\",\n        \"watch\"\n      ]\n    }\n  ]\n}\n```","ref":"discovery.html#using-the-file-driver-k8s-discovery-driver-file"},{"type":"extras","title":"Middleware (K8s.Middleware)","doc":"# Middleware (`K8s.Middleware`)\n\n`K8s.Middleware` is associated to a `K8s.Conn`. By default the following `K8s.Middleware.Stack` is associated to all connections:\n\n* `K8s.Middleware.Request.Initialize`\n* `K8s.Middleware.Request.EncodeBody`","ref":"middleware.html"},{"type":"extras","title":"Changing a Connection's Middleware - Middleware (K8s.Middleware)","doc":"Given a registered `K8s.Conn` named `\"foo\"` the following example will add `ExampleMiddleware` to the end of the middleware stack\n\n```elixir\n{:ok, conn} = K8s.Conn.from_service_account()\nnew_stack = K8s.Middleware.Stack{request: [MyFirstMiddleware, MySecondMiddleware]}\nconn_with_new_middleware = %K8s.Conn{conn | middleware: new_stack}\n```","ref":"middleware.html#changing-a-connection-s-middleware"},{"type":"extras","title":"Writing Middleware - Middleware (K8s.Middleware)","doc":"#","ref":"middleware.html#writing-middleware"},{"type":"extras","title":"Request Middleware - Middleware (K8s.Middleware)","doc":"`K8s.Middleware.Request` is a behaviour and struct for encapsulating requests processed by the middleware stack. \n\nMiddleware is expected to return `{:ok, %Request{}}` to continue processing or `{:error, :my_error}` to halt. The error in the error tuple can be an atom or a struct. It will automatically be wrapped in `K8s.Middleware.Error` during processing.\n\nTo implement a piece of middleware, you need to define a function `call/1` that accepts a `K8s.Middleware.Request`.\n\nThe example below will automatically add labels to all `:post` requests.\n\n```elixir\ndef call(%Request{method: :post, body: body} = req) do\n  metadata = Map.get(body, \"metadata\", %{})\n  metadata_with_labels = Map.put(metadata, \"labels\", %{\"env\" => \"prod\"})\n  updated_body = Map.put(body, \"metadata\", metadata_with_labels)\n\n  request_with_labels = %Request{req | body: updated_body}\n  {:ok, request_with_labels}\nend\n```\n\n#","ref":"middleware.html#request-middleware"},{"type":"extras","title":"Response Middleware - Middleware (K8s.Middleware)","doc":"Response middleware has not been implemented at this time.","ref":"middleware.html#response-middleware"},{"type":"extras","title":"Migrations","doc":"# Migrations","ref":"migrations.html"},{"type":"extras","title":"Migrating from `< 2.0.0` - Migrations","doc":"With version `2.0.0` some breaking changes were introduced in order to stabilize\nthe library. Most of them are in the `watch` and `stream` functionalities.\nIf you're using `K8s.Client.DynamicHTTPProvider` in your tests, you\nwill have to change your mocks slightly.\n\n#","ref":"migrations.html#migrating-from-2-0-0"},{"type":"extras","title":"DynamicHTTPProvider - Migrations","doc":"The spec for `K8s.Client.Provider.request/N` and `K8s.Client.Provider.stream/N`\nwere changed which reflects in your mock implementation if you're using\n`K8s.Client.DynamicHTTPProvider`. Concretely, the second variable is now of type\n`%URI{}` and contains the path and query_params.\n\n##","ref":"migrations.html#dynamichttpprovider"},{"type":"extras","title":"Migrating the path - Migrations","doc":"In most of the cases you're probably matching against the path that's passed\nto the function. In this case, you can just wrap the path by the struct:\n\n**Old:**\n\n```elixir\n  defmodule K8sMock do\n    def request(:get, \"/api/v1/namespaces\", _body, _headers, _opts) do\n      # code\n    end\n  end\n```\n\n**New:**\n\n```elixir\n  defmodule K8sMock do\n    def request(:get, %URI{path: \"/api/v1/namespaces\"}, _body, _headers, _opts) do\n      # code\n    end\n  end\n```\n\n##","ref":"migrations.html#migrating-the-path"},{"type":"extras","title":"Migrating the query parameters - Migrations","doc":"If you're matching against query parameters sent to the function, you will\nhave to match them in the `URI` as well:\n\n**Old:**\n\n```elixir\n  defmodule K8sMock do\n    def request(:get, \"/api/v1/namespaces\", _body, _headers, params: [labelSelector: \"app=nginx\"]) do\n      # code\n    end\n  end\n```\n\n**New:**\n\n```elixir\n  defmodule K8sMock do\n    def request(:get, %URI{path: \"/api/v1/namespaces\", query: \"labelSelector=app%3Dnginx\"}, _body, _headers, _opts) do\n      # code\n    end\n  end\n```\n\n#","ref":"migrations.html#migrating-the-query-parameters"},{"type":"extras","title":"Watch and Stream - Migrations","doc":"`K8s.Client.watch/N` was used to run a `:list` operation and watch it. With this\nrelease, there is now a `:watch` operation. `K8s.Client.watch/N` now is used\nto create a `:watch` operation which has to be passed to `K8s.Client.stream/N`.\nAlso, the `stream_to` option was removed. See below.\n\n#","ref":"migrations.html#watch-and-stream"},{"type":"extras","title":"Streaming to another process - The `stream_to` option was removed - Migrations","doc":"Before `2.0.0`, you could pass the `stream_to` option to stream packets to a\nprocess. With the migration from HTTPoison to Mint, this option was removed.\nFor `:connect` operations, `K8s.Client.stream_to/N` can be used as a replacement.\nOther operations will have to be streamed using `K8s.Client.stream/N` or go over\n`K8s.Client.Runner.Base.stream_to/N`.\n\n##","ref":"migrations.html#streaming-to-another-process-the-stream_to-option-was-removed"},{"type":"extras","title":"Examples - Migrations","doc":"```elixir\n:ok =\n  K8s.Client.watch(\"v1\", \"ConfigMap\", namespace: \"default\")\n  |> K8s.Client.put_conn(conn)\n  |> K8s.Client.Runner.Base.stream_to(self())\n\nreceive do\n  event -> IO.inspect(event)\nend\n```\n\nPass a `{pid, reference}` tuple to get scoped messages:\n\n```elixir\nref = make_ref()\n:ok =\n  K8s.Client.watch(\"v1\", \"ConfigMap\", namespace: \"default\")\n  |> K8s.Client.put_conn(conn)\n  |> K8s.Client.Runner.Base.stream_to({self(), ref})\n\nreceive do\n  {^ref, event} -> IO.inspect(event)\nend\n```\n\n\n#","ref":"migrations.html#examples"},{"type":"extras","title":"Local Clusters and TLS Hostname Verification - Migrations","doc":"Local clusters are usually accessed via IP address so the hostname provided by\nthe TLS certificate can't match the actual hostname (IP address). While somehow\nHTTPoison was OK with this, Mint declines the HTTPS connection. In order to make\nthis work, you have to disable TLS peer verification. See also issue\n[#203](https://github.com/coryodaniel/k8s/issues/203)\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(...)\nconn = struct!(conn, insecure_skip_tls_verify: true)\n```\n\n#","ref":"migrations.html#local-clusters-and-tls-hostname-verification"},{"type":"extras","title":"HTTPoison was removed - Migrations","doc":"Wherever you match against `HTTPoison.*`, you're gonna have to change your code.\nThis might be the case in your error handlers. With `2.0.0`, all HTTP errors are\nwrapped in a `K8s.Client.HTTPError` exception struct.\n\n```\n\n```","ref":"migrations.html#httpoison-was-removed"},{"type":"extras","title":"Observability","doc":"# Observability","ref":"observability.html"},{"type":"extras","title":"Logging - Observability","doc":"All logs produced by this library should have `library: :k8s` as metadata which can be used in filters. \n`K8s.Sys.Logger` watches all telemetry events and logs them with `:measurements` and `:metadata` \nmetadata entries which can be used in structured logs. In order to use it, you have to attach its \ntelemetry handler. Call `K8s.Sys.Logger.attach/0` in your `application.ex`:\n\n```elixir\nK8s.Sys.Logger.attach()\n```","ref":"observability.html#logging"},{"type":"extras","title":"Telemetry - Observability","doc":"`K8s.Sys.Telemetry` provides a list of events _executed_ by this library. Use it to attach handlers to\nthem.","ref":"observability.html#telemetry"},{"type":"extras","title":"Tracing - Observability","doc":"Tracing has not been implemented yet.","ref":"observability.html#tracing"},{"type":"extras","title":"HTTP Operations (K8s.Operation)","doc":"# HTTP Operations (`K8s.Operation`)\n\n`K8s.Operation`s are Kubernetes REST operations. They encapsulate all the details of an HTTP request _except_ the server to perform them against.\n\nMany more client examples exist in the `K8s.Client` docs.","ref":"operations.html"},{"type":"extras","title":"Creating a Deployment from a Map - HTTP Operations (K8s.Operation)","doc":"```elixir\nresource = %{\n  \"apiVersion\" => \"apps/v1\",\n  \"kind\" => \"Deployment\",\n  \"metadata\" => %{\n    \"labels\" => %{\"app\" => \"nginx\"},\n    \"name\" => \"nginx-deployment\",\n    \"namespace\" => \"default\"\n  },\n  \"spec\" => %{\n    \"replicas\" => 3,\n    \"selector\" => %{\"matchLabels\" => %{\"app\" => \"nginx\"}},\n    \"template\" => %{\n      \"metadata\" => %{\"labels\" => %{\"app\" => \"nginx\"}},\n      \"spec\" => %{\n        \"containers\" => [\n          %{\n            \"image\" => \"nginx:1.7.9\",\n            \"name\" => \"nginx\",\n            \"ports\" => [%{\"containerPort\" => 80}]\n          }\n        ]\n      }\n    }\n  }\n}\n\noperation = K8s.Client.create(resource)\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n{:ok, response} = K8s.Client.run(conn, operation)\n```","ref":"operations.html#creating-a-deployment-from-a-map"},{"type":"extras","title":"Creating a Deployment from a YAML File - HTTP Operations (K8s.Operation)","doc":"`K8s.Resource` provides YAML resource parsing and interpolation support as well as a few helper functions for accessing common Kubernetes resource fields.\n\nGiven the YAML file `priv/deployment.yaml`:\n\n```yaml\napiVersion: apps/v1\nkind: Deployment\nmetadata:\n  name: <%= name %>-deployment\n  namespace: <%= namespace %>\n  labels:\n    app: <%= name %>\nspec:\n  replicas: 3\n  selector:\n    matchLabels:\n      app: <%= name %>\n  template:\n    metadata:\n      labels:\n        app: <%= name %>\n    spec:\n      containers:\n        - name: <%= name %>\n          image: <%= image %>\n          ports:\n            - containerPort: 80\n```\n\n```elixir\nopts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\nresource = K8s.Resource.from_file!(\"priv/deployment.yaml\", opts)\n\noperation = K8s.Client.create(resource)\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n{:ok, deployment} = K8s.Client.run(conn, operation)\n```","ref":"operations.html#creating-a-deployment-from-a-yaml-file"},{"type":"extras","title":"Listing Deployments - HTTP Operations (K8s.Operation)","doc":"In a given namespace:\n\n```elixir\noperation = K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: \"prod\")\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n{:ok, deployments} = K8s.Client.run(conn, operation)\n```\n\nAcross all namespaces:\n\n```elixir\noperation = K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: :all)\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n{:ok, deployments} = K8s.Client.run(conn, operation)\n```\n\n_Note:_ `K8s.Client.list` will return a `map`. The list of resources will be under `\"items\"`.","ref":"operations.html#listing-deployments"},{"type":"extras","title":"Using `labelSelector` with List Operations - HTTP Operations (K8s.Operation)","doc":"`K8s.Selector` supports programatically building Kubernetes `labelSelector`s.\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\noperation =\n  K8s.Client.list(\"apps/v1\", :deployments)\n  |> K8s.Selector.label({\"app\", \"nginx\"})\n  |> K8s.Selector.label_in({\"environment\", [\"qa\", \"prod\"]})\n\nK8s.Client.run(conn, operation)\n```","ref":"operations.html#using-labelselector-with-list-operations"},{"type":"extras","title":"Getting a Deployment - HTTP Operations (K8s.Operation)","doc":"```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\noperation = K8s.Client.get(\"apps/v1\", :deployment, [namespace: \"default\", name: \"nginx-deployment\"])\n{:ok, deployment} = K8s.Client.run(conn, operation)\n```","ref":"operations.html#getting-a-deployment"},{"type":"extras","title":"Watch Operations (`K8s.Client.Runner.Watch`) - HTTP Operations (K8s.Operation)","doc":"Watch operations use the Kubernetes Watch API to stream `added`, `modified`, and `deleted` as they occur.\n\nTo get a stream of events:\n\n```elixir\noperation = K8s.Client.watch(\"apps/v1\", :deployment, namespace: :all)\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n{:ok, event_stream} = K8s.Client.stream(conn, operation)\n```","ref":"operations.html#watch-operations-k8s-client-runner-watch"},{"type":"extras","title":"Wait on a Resource (`K8s.Client.Runner.Wait`) - HTTP Operations (K8s.Operation)","doc":"The wait runner permits read operations to be made and block until a certain state is met in Kubernetes.\n\nThis follow example will wait 60 seconds for the field `status.succeeded` to equal `1`.\n\n```elixir\noperation = K8s.Client.get(\"batch/v1\", :job, namespace: \"default\", name: \"database-migrator\")\nwait_opts = [find: [\"status\", \"succeeded\"], eval: 1, timeout: 60]\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n{:ok, job} = K8s.Client.wait_until(conn, operation, wait_opts)\n```\n\n`:find` and `:eval` also accept functions to apply to check success.","ref":"operations.html#wait-on-a-resource-k8s-client-runner-wait"},{"type":"extras","title":"Async Batch Operations (`K8s.Client.Runner.Async`) - HTTP Operations (K8s.Operation)","doc":"An async runner is provided for running operations in parallel. All operations are fired async and their results are returned. Processing does not halt if an error occurs for one operation.\n\n```elixir\noperation1 = K8s.Client.get(\"v1\", \"Pod\", namespace: \"default\", name: \"pod-1\")\noperation2 = K8s.Client.get(\"v1\", \"Pod\", namespace: \"default\", name: \"pod-2\")\n\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\nresults = K8s.Client.async(conn, [operation1, operation2])\n```\n\n`results` will be a list of `:ok` and `:error` tuples.","ref":"operations.html#async-batch-operations-k8s-client-runner-async"},{"type":"extras","title":"List Operations as Elixir Streams (`K8s.Client.Runner.Stream`) - HTTP Operations (K8s.Operation)","doc":"A stream runner is provided to automatically handle pagination in `K8s.Client.list/3` operations.\n\n```elixir\noperation = K8s.Client.list(\"v1\", \"Pod\", namespace: :all)\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\nconn\n|> K8s.Client.stream(operation)\n|> Stream.filter(&my_filter_function?/1)\n|> Stream.map(&my_map_function?/1)\n|> Enum.into([])\n```","ref":"operations.html#list-operations-as-elixir-streams-k8s-client-runner-stream"},{"type":"extras","title":"Connect to `pods/exec` subresource and execute commands - HTTP Operations (K8s.Operation)","doc":"Use the `:connect` operation to connect to the `pods/exec` subresource and\nexecute commands. A `:connect` operation is created with `K8s.Client.connect/N`.\nWhen connecting to `pods/exec`, be sure to pass the command you want to run in\nthe options.\n\n#","ref":"operations.html#connect-to-pods-exec-subresource-and-execute-commands"},{"type":"extras","title":"Waiting for command termination - HTTP Operations (K8s.Operation)","doc":"If you want to run a command that terminates and wait for it, pass the `:connect`\noperation to `K8s.Client.run/N`.\n\n```elixir\n  {:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\n\n  op = K8s.Client.connect(\n    \"v1\",\n    \"pods/exec\",\n    [namespace: \"default\", name: \"nginx-8f458dc5b-zwmkb\"],\n    command: [\"/bin/sh\", \"-c\", \"nginx -t\"]\n  )\n\n  {:ok, response} = K8s.Client.run(conn, op)\n```\n\n#","ref":"operations.html#waiting-for-command-termination"},{"type":"extras","title":"Opening long-lasting connections (e.g. a shell) and sending messages to pods - HTTP Operations (K8s.Operation)","doc":"If you send a command that does not terminate (e.g. `/bin/sh`) or one that takes\nlong to terminate, you can use `K8s.Client.stream_to/3` which opens a connection\nin a separate process and sends messages to your pid. The function returns a\ncallback that you can use to send messages to the pod.\n\nThe following is a very simple example. In your code you'd rather use a\n`GenServer` or so to track the state.\n\n```elixir\ndefmodule TestPodExec do\n  require Logger\n\n  def recv_loop() do\n    receive do\n      {:open, true} ->\n        Logger.debug(\"Connection established.\")\n        recv_loop()\n\n      {:stdout, msg} ->\n        Logger.info(~s(Message received on stdout: \"#{msg}\"))\n        recv_loop()\n\n      {:close, reason} ->\n        Logger.debug(\"Connection closed. Reason: #{inspect(reason)}\")\n    end\n  end\n\n  def run() do\n    {:ok, pid} =\n      Task.start_link(fn ->\n        recv_loop()\n      end)\n\n    {:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\n\n    op = K8s.Client.connect(\n      \"v1\",\n      \"pods/exec\",\n      [namespace: \"default\", name: \"nginx-8f458dc5b-zwmkb\"],\n      command: [\"/bin/sh\"],\n      tty: true\n    )\n\n    {:ok, send_to_pod} = K8s.Client.stream_to(conn, op, pid)\n\n    Process.sleep(1000)\n    send_to_pod.({:stdin, ~s(echo \"hello world\"\\n)})\n\n    Process.sleep(1000)\n    send_to_pod.(:close)\n\n    Process.sleep(1000)\n  end\nend\n\nTestPodExec.run()\n```\n\nWhat you should see:\n\n- A DEBUG log stmt saying `Connection established.`\n- A couple of stdout messages. It is not deterministic whether the pod sends\n  them in one chunk or in multiple:\n  - Shell prompt (e.g. `# ` or `$ `)\n  - Unless we opt out of it (`stty -echo`), the pod sends back what we \"type\":\n    `echo \"hello world\"\\n`\n  - The actual echo: `hello world`\n  - The next shell prompt\n- A DEBUG log stmt saying `Connection closed.`\n\nIf we concatenate all the stdout messages you see it reads what you would\nsee on a shell:\n\n```\n$ echo \"hello world\"\nhello world\n$\n```\n\n#","ref":"operations.html#opening-long-lasting-connections-e-g-a-shell-and-sending-messages-to-pods"},{"type":"extras","title":"Options - HTTP Operations (K8s.Operation)","doc":"- `command` - required for running commands\n- `container` - if a pod runs multiple containers, you have to specify the container to run the command in.\n- `stdin` - enable stdin (defaults to `true`)\n- `stdout` - enable stdout (defaults to `true`)\n- `stderr` - enable stderr (defaults to `true`)\n- `tty` - stdin is a TTY (defaults to `false`)\n\n```elixir\n  {:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\n\n  op = K8s.Client.connect(\n    \"v1\",\n    \"pods/exec\",\n    [namespace: \"default\", name: \"nginx-8f458dc5b-zwmkb\"],\n    command: [\"/bin/sh\", \"-c\", \"nginx -t\"],\n    container: \"main\",\n    tty: true\n  )\n\n  {:ok, response} = K8s.Client.run(conn, op)\n```","ref":"operations.html#options"},{"type":"extras","title":"Connect to `pods/log` subresource to read logs from Pods - HTTP Operations (K8s.Operation)","doc":"Use the `:connect` operation to connect to the `pods/log` subresource. A\n`:connect` operation is created with `K8s.Client.connect/N`.\n\n#","ref":"operations.html#connect-to-pods-log-subresource-to-read-logs-from-pods"},{"type":"extras","title":"Options - HTTP Operations (K8s.Operation)","doc":"Refer to the [Kubernetes\ndocumentation](https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.27/#read-log-pod-v1-core)\nfor documentation on these options.\n\n- `container`\n- `follow` - Use with `K8s.Client.stream/N` or `K8s.Client.stream_to/N`.\n- `insecureSkipTLSVerifyBackend`\n- `limitBytes`\n- `pretty`\n- `previous`\n- `sinceSeconds`\n- `tailLines`\n- `timestamps`\n\n```elixir\n  {:ok, conn} = K8s.Conn.from_file(\"~/.kube/config\")\n\n  {:ok, stream} = K8s.Client.connect(\n    \"v1\",\n    \"pods/log\",\n    [namespace: \"default\", name: \"nginx-8f458dc5b-zwmkb\"],\n    command: [\"/bin/sh\", \"-c\", \"nginx -t\"],\n    container: \"main\",\n    follow: true\n  )\n  |> K8s.Client.put_conn(conn)\n  |> K8s.Client.stream()\n```","ref":"operations.html#options"},{"type":"extras","title":"Testing","doc":"# Testing\n\n`k8s` supports a few mechanisms to stub Kubernetes HTTP requests when developing or testing locally.","ref":"testing.html"},{"type":"extras","title":"Mocking API Version and Resource Discovery - Testing","doc":"`K8s` does resource discovery of the Kubernetes API before running `K8s.Operation`s. When testing if a Kubernetes cluster isn't available you can use the module `K8s.Discovery.Driver.File` to stub Kubernetes `api-resource` and `api-versions` responses.\n\nThe mock used in the test suite is [here](./test/support/discovery/example.json).\n\nThe driver can be set for all clusters or per-cluster:\n\n```elixir\nconfig :k8s,\n  discovery_driver: K8s.Discovery.Driver.File,\n  discovery_opts: [config: \"test/support/discovery/example.json\"]\n```\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"....\")\nconn = %K8s.Conn{conn | discovery_driver: AlternateDriver}\n```","ref":"testing.html#mocking-api-version-and-resource-discovery"},{"type":"extras","title":"Mocking HTTP Responses of `K8s.Operations` in your application - Testing","doc":"`K8s` ships with a [`K8s.Client.DynamicHTTPProvider`](./lib/k8s/client/dynamic_http_provider.ex) for stubbing HTTP responses to kubernetes API requests.\n\nThis provider is used throughout the test suite for mocking HTTP responses.\n\nTo enable the dynamic HTTP provider it must be turned on in your `config.exs`:\n\n```elixir\nconfig :k8s, http_provider: K8s.Client.DynamicHTTPProvider\n```\n\n```elixir\ndefmodule MyApp.ResourceTest do\n  use ExUnit.Case, async: true\n\n  defmodule K8sMock do\n    @namespaces_path \"/api/v1/namespaces\"\n\n    def request(:get, %URI{path: @namespaces_path}, _, _, _) do\n      namespaces = [%{\"metadata\" => %{\"name\" => \"default\"}}]\n      body = Jason.encode!(namespaces)\n      {:ok, %HTTPoison.Response{status_code: 200, body: body}}\n    end\n  end\n\n  setup do\n    DynamicHTTPProvider.register(self(), __MODULE__.K8sMock)\n  end\n\n  test \"gets namespaces\" do\n    conn = %K8s.Conn{} # set up your conn\n    operation = K8s.Client.get(\"v1\", :namespaces)\n    assert {:ok, namespaces} = K8s.Client.run(conn, operation)\n    assert namespaces == [%{\"metadata\" => %{\"name\" => \"default\"}}]\n  end\nend\n```\n\nTo see advanced examples of usage, check out these examples in the test suite:\n\n- [client/runner/base](../test/k8s/client/runner/base_test.exs)\n- [client/runner/stream](../test/k8s/client/runner/stream_test.exs)\n- [client/runner/stream/watch](../test/k8s/client/runner/stream/watch_test.exs)\n- [discovery](../test/k8s/discovery_test.exs)","ref":"testing.html#mocking-http-responses-of-k8s-operations-in-your-application"},{"type":"extras","title":"Integration Testing - Testing","doc":"A `Makefile` is included for help with integration testing against k3s.\n\nRun `make help` for a list of commands:\n\n```\ntest.integration               Run integration tests using k3d `make cluster`\ntest.watch                     Run all tests with mix.watch\ntest                           Run all tests\n```\n\n#","ref":"testing.html#integration-testing"},{"type":"extras","title":"Integration environment variables - Testing","doc":"- `KUBECONFIG` path to kubeconfig file for integration tests, default: \"./integration.yaml\"\n- `TEST_WAIT_TIMEOUT` number of seconds to timeout running Wait integration tests, default: 5","ref":"testing.html#integration-environment-variables"},{"type":"extras","title":"Usage","doc":"# Usage\n\n- [Connections (`K8s.Conn`)](./connections.md)\n- [Operations (`K8s.Operation`)](./operations.md)\n- [Discovery (`K8s.Discovery`)](./discovery.md)\n- [Middleware (`K8s.Middleware`)](./middleware.md)\n- [Authentication (`K8s.Conn.Auth`)](./authentication.md)\n- [Observability](./observability.md)\n- [Testing](./testing.md)\n- [Advanced Topics](./advanced.md) - CRDs, Multiple Clusters, and Subresource Requests\n- [Migrations](./migrations.md) - Migrating from earlier versions","ref":"usage.html"},{"type":"extras","title":"tl;dr Examples - Usage","doc":"#","ref":"usage.html#tl-dr-examples"},{"type":"extras","title":"Creating a deployment - Usage","doc":"```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\nopts = [namespace: \"default\", name: \"nginx\", image: \"nginx:nginx:1.7.9\"]\n{:ok, resource} = K8s.Resource.from_file(\"priv/deployment.yaml\", opts)\n\n{:ok, deployment} =\n    resource\n    |> K8s.Client.create()\n    |> K8s.Client.put_conn(conn)\n    |> K8s.Client.run()\n```\n\n#","ref":"usage.html#creating-a-deployment"},{"type":"extras","title":"Listing deployments - Usage","doc":"In a namespace:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\n{:ok, deployments} =\n    K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: \"prod\")\n    |> K8s.Client.put_conn(conn)\n    |> K8s.Client.run()\n```\n\nAcross all namespaces:\n\n```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\n{:ok, deployments} =\n    K8s.Client.list(\"apps/v1\", \"Deployment\", namespace: :all)\n    |> K8s.Client.put_conn(conn)\n    |> K8s.Client.run()\n```\n\n#","ref":"usage.html#listing-deployments"},{"type":"extras","title":"Getting a deployment - Usage","doc":"```elixir\n{:ok, conn} = K8s.Conn.from_file(\"path/to/kubeconfig.yaml\")\n\n{:ok, deployment} =\n    K8s.Client.get(\"apps/v1\", :deployment, [namespace: \"default\", name: \"nginx-deployment\"])\n    |> K8s.Client.put_conn(conn)\n    |> K8s.Client.run(conn, operation)\n```","ref":"usage.html#getting-a-deployment"}],"content_type":"text/markdown"}